<?php
 namespace App\Controller\Frontend; use Symfony\Component\HttpFoundation\Request; use Symfony\Component\HttpFoundation\Response; use Symfony\Component\Form\Form; use Symfony\Component\Form\Extension\Core\Type\SubmitType; use Symfony\Contracts\Translation\TranslatorInterface; use Symfony\Component\Validator\Validator\ValidatorInterface; use Symfony\Component\Validator\ConstraintViolationList; use App\Controller\Controller; use App\Event\EventQueue; use App\Service\Logger; use App\Entity\Site as SiteEntity; use App\Entity\Certificate as CertificateEntity; use App\Entity\NodejsSettings as NodejsSettingsEntity; use App\Entity\PhpSettings as PhpSettingsEntity; use App\Entity\PythonSettings as PythonSettingsEntity; use App\Entity\Database as DatabaseEntity; use App\Entity\DatabaseUser as DatabaseUserEntity; use App\Entity\DatabaseServer as DatabaseServerEntity; use App\Entity\User as UserEntity; use App\Entity\Manager\ConfigManager; use App\Entity\Manager\SiteManager as SiteEntityManager; use App\Entity\Manager\CertificateManager as CertificateEntityManager; use App\Entity\Manager\DatabaseManager as DatabaseEntityManager; use App\Entity\Manager\DatabaseUserManager as DatabaseUserEntityManager; use App\Entity\Manager\DatabaseServerManager as DatabaseServerEntityManager; use App\Entity\Manager\VhostTemplateManager as VhostTemplateEntityManager; use App\Database\Connection as DatabaseConnection; use App\Database\Manager as DatabaseManager; use App\Site\Nginx\Vhost\StaticTemplate as StaticVhostTemplate; use App\Site\Nginx\Vhost\PhpTemplate as PhpVhostTemplate; use App\Site\Nginx\Vhost\PythonTemplate as PythonVhostTemplate; use App\Site\Nginx\Vhost\ReverseProxyTemplate as ReverseProxyVhostTemplate; use App\Site\Nginx\Vhost\Processor\RedirectDomain as RedirectDomainProcessor; use App\Site\Nginx\Vhost\Processor\RedirectServerName as RedirectServerNameProcessor; use App\Site\Nginx\Vhost\Processor\ServerName as ServerNameProcessor; use App\Site\Parser\DomainName as DomainNameParser; use App\Site\Ssl\Generator\RsaKeyGenerator; use App\Site\Creator\NodejsSite as NodejsSiteCreator; use App\Site\Creator\StaticSite as StaticSiteCreator; use App\Site\Creator\PhpSite as PhpSiteCreator; use App\Site\Creator\PythonSite as PythonSiteCreator; use App\Site\Creator\ReverseProxySite as ReverseProxySiteCreator; use App\Site\Ssl\DistinguishedName; use App\Site\Ssl\Generator\CsrGenerator; use App\Site\Ssl\Util\Openssl; use App\Site\NodejsSite; use App\Site\StaticSite; use App\Site\PhpSite; use App\Site\PythonSite; use App\Site\ReverseProxySite; use App\Util\PasswordGenerator; use App\Site\Application\WordPressInstaller; use App\Service\Crypto; class NewSiteController extends Controller { private const WORDPRESS_VHOST_TEMPLATE_NAME = "\127\x6f\x72\144\120\162\145\x73\163"; private const NODEJS_VHOST_TEMPLATE_NAME = "\116\157\x64\x65\152\x73"; private const STATIC_SITE_VHOST_TEMPLATE_NAME = "\x53\x74\141\x74\151\x63"; private const PYTHON_VHOST_TEMPLATE_NAME = "\x50\x79\164\x68\x6f\x6e"; private const REVERSE_PROXY_VHOST_TEMPLATE_NAME = "\x52\x65\166\145\162\163\x65\120\x72\x6f\170\x79"; private const VARNISH_CACHE_SERVER = "\61\62\67\56\x30\x2e\x30\x2e\61\72\66\60\x38\61"; private const PASSWORD_LENGTH = 20; private ConfigManager $configManager; private SiteEntityManager $siteEntityManager; private CertificateEntityManager $certificateEntityManager; private DatabaseEntityManager $databaseEntityManager; private DatabaseUserEntityManager $databaseUserEntityManager; private DatabaseServerEntityManager $databaseServerEntityManager; private DomainNameParser $domainNameParser; private VhostTemplateEntityManager $vhostTemplateEntityManager; private ValidatorInterface $validator; private array $wordPressDefaultConfigValues = ["\106\123\x5f\115\105\x54\110\117\x44" => ["\x76\141\154\165\145" => "\x64\x69\162\x65\143\164", "\x72\141\x77" => false], "\127\120\x5f\x44\105\x42\x55\x47\137\104\x49\x53\120\114\101\131" => ["\166\x61\154\x75\x65" => "\x66\141\154\x73\145", "\162\141\x77" => true], "\x57\x50\x5f\104\105\102\x55\107\x5f\x4c\x4f\107" => ["\x76\141\154\x75\x65" => "\x74\162\x75\x65", "\162\x61\167" => true], "\103\x4f\x4e\103\101\124\x45\x4e\101\124\105\137\123\x43\122\111\x50\124\123" => ["\166\x61\x6c\x75\x65" => "\146\141\x6c\x73\x65", "\162\141\167" => true], "\x41\125\124\x4f\x53\x41\126\x45\x5f\x49\116\124\x45\x52\126\x41\x4c" => ["\x76\141\x6c\x75\x65" => 600, "\x72\x61\x77" => true], "\x57\x50\x5f\x50\117\x53\x54\137\x52\x45\x56\111\x53\111\117\116\x53" => ["\x76\x61\154\x75\145" => 5, "\x72\x61\x77" => true], "\105\x4d\x50\x54\x59\x5f\x54\x52\101\x53\x48\x5f\104\101\x59\123" => ["\x76\x61\x6c\165\x65" => 21, "\x72\x61\x77" => true]]; private array $wordPressMultiSiteConfigValues = ["\x57\x50\x5f\101\x4c\x4c\x4f\127\x5f\x4d\125\x4c\x54\x49\x53\111\124\x45" => ["\x76\141\154\x75\x65" => "\164\x72\x75\x65", "\162\141\167" => true], "\115\x55\114\x54\x49\x53\x49\124\105" => ["\166\141\154\165\145" => "\x74\162\x75\145", "\162\141\x77" => true], "\123\125\102\x44\x4f\x4d\101\111\x4e\137\111\x4e\123\x54\101\x4c\x4c" => ["\166\x61\154\x75\145" => "\146\141\154\163\x65", "\x72\141\x77" => true], "\x44\117\x4d\101\x49\x4e\x5f\103\125\x52\x52\x45\x4e\124\x5f\123\111\124\x45" => ["\x76\141\x6c\165\145" => '', "\162\x61\x77" => false], "\120\101\124\110\x5f\x43\x55\x52\122\x45\116\x54\137\x53\x49\124\105" => ["\x76\141\154\x75\145" => "\57", "\x72\x61\x77" => false], "\x53\x49\124\x45\137\x49\104\x5f\103\125\x52\x52\105\116\x54\x5f\x53\x49\124\105" => ["\x76\x61\154\x75\x65" => 1, "\162\141\x77" => true], "\x42\114\x4f\x47\137\111\x44\x5f\x43\x55\122\122\105\116\x54\x5f\x53\111\x54\105" => ["\166\x61\154\165\145" => 1, "\x72\141\167" => true]]; public function __construct(ConfigManager $configManager, SiteEntityManager $siteEntityManager, CertificateEntityManager $certificateEntityManager, DatabaseEntityManager $databaseEntityManager, DatabaseUserEntityManager $databaseUserEntityManager, DatabaseServerEntityManager $databaseServerEntityManager, VhostTemplateEntityManager $vhostTemplateEntityManager, DomainNameParser $domainNameParser, ValidatorInterface $validator, TranslatorInterface $translator, Logger $logger) { goto cea6b; f77cf: $this->siteEntityManager = $siteEntityManager; goto B0b19; cea6b: $this->configManager = $configManager; goto f77cf; F9623: $this->databaseEntityManager = $databaseEntityManager; goto cb053; Cfeda: $this->domainNameParser = $domainNameParser; goto B11e6; C4e6a: $this->vhostTemplateEntityManager = $vhostTemplateEntityManager; goto Cfeda; cb053: $this->databaseUserEntityManager = $databaseUserEntityManager; goto d7b1b; d7b1b: $this->databaseServerEntityManager = $databaseServerEntityManager; goto C4e6a; d4dc6: parent::__construct($translator, $logger); goto F2c4f; B11e6: $this->validator = $validator; goto d4dc6; B0b19: $this->certificateEntityManager = $certificateEntityManager; goto F9623; F2c4f: } public function index(Request $request) : Response { goto B38ed; B3def: return $response; goto Ac610; D70c3: if (true === $hasPermissions) { goto D9bee; } goto c8316; D64a1: D9bee: goto A3842; B38ed: $hasPermissions = $this->hasPermissions(); goto D70c3; c8316: $response = $this->redirect($this->generateUrl("\143\154\160\137\163\x69\x74\x65\163")); goto b9cc4; b9cc4: goto fe5ad; goto D64a1; De993: fe5ad: goto B3def; bb8f2: $response = $this->render("\106\162\x6f\156\x74\145\156\x64\x2f\123\x69\164\x65\57\116\145\x77\57\151\156\144\145\170\x2e\150\x74\155\154\x2e\x74\167\x69\x67", ["\x75\x73\145\x72" => $user]); goto De993; A3842: $user = $this->getUser(); goto bb8f2; Ac610: } public function newPythonSite(Request $request) : Response { goto c4e42; E8aef: $response = $this->handleCreatePythonSiteForm($request, $form); goto Ec2c3; e32b2: $response = $this->render("\106\162\x6f\156\x74\x65\156\144\x2f\123\x69\164\x65\x2f\116\x65\167\x2f\x70\x79\x74\150\157\x6e\x2e\x68\x74\155\x6c\x2e\164\x77\x69\x67", ["\x75\163\145\162" => $user, "\x66\157\162\155" => $form->createView(), "\146\x6f\x72\x6d\x45\x72\x72\x6f\x72\x73" => $this->formErrors]); goto afb0a; f8959: $user = $this->getUser(); goto Ef13a; A6bf8: $form = $this->createPythonSiteForm($pythonSettings); goto ded31; a03d3: $response = $this->redirect($this->generateUrl("\143\x6c\x70\137\163\151\164\x65\163")); goto a5e5f; d939f: C812b: goto e32b2; b65ac: $form->handleRequest($request); goto E263a; B975f: if (true === $hasPermissions) { goto E35d0; } goto a03d3; a5e5f: goto c9555; goto eb93e; A8601: return $response; goto d671c; E263a: if (!(true === $form->isSubmitted())) { goto F60bd; } goto E8aef; f6aeb: F60bd: goto d939f; Ec2c3: if (!(false === is_null($response))) { goto Ca901; } goto A8601; c4e42: $hasPermissions = $this->hasPermissions(); goto B975f; ded31: if (!(true === $request->isMethod("\120\117\123\x54"))) { goto C812b; } goto b65ac; Ef13a: $pythonSettings = new PythonSettingsEntity(); goto A6bf8; b5810: return $response; goto Ac680; d671c: Ca901: goto f6aeb; afb0a: c9555: goto b5810; eb93e: E35d0: goto f8959; Ac680: } private function createPythonSiteForm(PythonSettingsEntity $pythonSettings) : Form { goto c4cbd; c4cbd: $form = $this->createForm("\x41\x70\x70\134\106\157\x72\155\134\123\x69\164\x65\x4e\x65\167\120\x79\x74\x68\x6f\156\124\x79\x70\x65", $pythonSettings, ["\x61\x63\x74\151\x6f\x6e" => $this->generateUrl("\143\154\160\x5f\163\151\x74\145\x5f\x70\171\x74\x68\x6f\156\x5f\x6e\145\167"), "\155\x65\164\150\x6f\144" => "\x50\x4f\123\x54", "\x61\x74\164\162" => ["\x69\144" => "\x6e\145\167\55\x70\x79\x74\150\157\x6e\55\x73\x69\x74\145\55\146\x6f\162\155"]]); goto Ba795; Ba795: $form->add("\x73\x75\x62\x6d\x69\164", SubmitType::class, ["\x61\x74\164\162" => ["\143\154\141\163\163" => "\142\164\156\x20\x62\x74\x6e\x2d\x6c\147\40\142\x74\x6e\55\x62\x6c\165\145"], "\154\141\142\x65\154" => "\x43\x72\x65\141\164\x65"]); goto F9a8e; F9a8e: return $form; goto D383a; D383a: } private function handleCreatePythonSiteForm(Request $request, Form $form) { goto b4cfc; f4b1b: $this->formErrors = $this->getErrorMessages($form); goto Adcca; Adcca: goto B93e8; goto Fec47; e118f: B93e8: goto Ad963; b4cfc: if (true === $form->isValid()) { goto C21cf; } goto f4b1b; Fec47: C21cf: goto E62a7; E62a7: try { goto B1f02; Cb17c: return $response; goto F4ee4; f048b: if (false === is_null($vhostTemplateEntity)) { goto F10f3; } goto ff497; e7526: $vhostTemplateName = self::PYTHON_VHOST_TEMPLATE_NAME; goto a6951; D091c: goto Cc507; goto Feccf; Eadf3: $session->getFlashBag()->set("\163\x75\x63\143\145\163\163", $this->translator->trans("\123\x69\x74\145\x20\150\141\x73\x20\x62\x65\145\156\x20\143\x72\145\141\x74\145\x64\x2e")); goto adef2; B3775: $domainName = $form->get("\x64\x6f\155\x61\151\x6e\x4e\x61\155\x65")->getData(); goto D3159; b636b: $eventData = ["\x73\151\164\x65" => $domainName, "\163\151\x74\145\125\163\x65\x72" => $siteUser, "\160\171\164\x68\157\156\x56\145\162\163\x69\157\156" => $pythonVersion, "\160\x6f\162\164" => $port]; goto ecb4b; Feccf: F10f3: goto E8272; dfe1a: $siteUserPassword = $form->get("\163\x69\164\x65\125\163\145\x72\x50\141\x73\x73\x77\x6f\162\x64")->getData(); goto e7526; c7f0d: $port = $form->get("\x70\x6f\x72\x74")->getData(); goto F1dc1; F4ee4: Cc507: goto B3eba; B1f02: $session = $request->getSession(); goto B3775; F1dc1: $siteUser = $form->get("\163\x69\x74\x65\125\163\145\x72")->getData(); goto dfe1a; ff497: throw new \Exception(sprintf("\x56\x68\157\x73\x74\x20\x74\145\x6d\x70\x6c\x61\164\145\40\x22\45\x73\x22\40\156\157\164\40\x66\x6f\x75\x6e\x64\56", $vhostTemplateName)); goto D091c; a6951: $vhostTemplateEntity = $this->getVhostTemplateEntity($vhostTemplateName); goto f048b; ef38e: $this->createPythonSite($domainName, $pythonVersion, $port, $siteUser, $siteUserPassword); goto b636b; E8272: $user = $this->getUser(); goto ef38e; adef2: $response = $this->redirect($this->generateUrl("\x63\x6c\x70\x5f\163\x69\x74\x65\x73")); goto Cb17c; D3159: $pythonVersion = $form->get("\x70\x79\164\x68\x6f\156\126\x65\162\163\151\x6f\156")->getData(); goto c7f0d; ecb4b: EventQueue::addEvent(EventQueue::EVENT_SITE_PYTHON_CREATE, $user, $eventData, $request); goto Eadf3; B3eba: } catch (\Exception $e) { $this->logger->exception($e); $session->getFlashBag()->set("\144\141\x6e\x67\x65\x72", $this->translator->trans("\x41\156\x20\x65\162\162\x6f\162\x20\150\141\x73\x20\157\x63\143\165\162\162\145\144\x2c\x20\145\162\162\157\162\x20\x6d\x65\x73\x73\141\147\145\x3a\40\45\x65\162\162\157\162\115\x65\x73\x73\x61\147\145\45", ["\45\145\162\162\157\x72\x4d\x65\163\x73\141\147\x65\x25" => $e->getMessage()])); } goto e118f; Ad963: } public function newReverseProxy(Request $request) : Response { goto Cbac8; A0ea9: if (!(true === $form->isSubmitted())) { goto D11ed; } goto Deeaf; Fa23a: return $response; goto da388; A3ba7: ddb0e: goto e13b5; A6c67: D11ed: goto f33e9; c163d: if (!(false === is_null($response))) { goto D0f7f; } goto Fa23a; a3e44: Fa633: goto Dd86a; Ab414: $form->handleRequest($request); goto A0ea9; Dd86a: $user = $this->getUser(); goto c8a00; a54a2: if (!(true === $request->isMethod("\x50\117\123\x54"))) { goto d6ed8; } goto Ab414; d11e7: $response = $this->render("\106\162\157\156\164\145\x6e\x64\57\123\151\164\145\x2f\x4e\145\167\57\x72\x65\166\x65\x72\163\x65\55\x70\x72\x6f\x78\x79\56\150\164\155\x6c\56\x74\167\151\147", ["\x75\x73\x65\x72" => $user, "\x66\157\x72\x6d" => $form->createView(), "\x66\x6f\162\x6d\x45\x72\x72\x6f\162\x73" => $this->formErrors]); goto A3ba7; f33e9: d6ed8: goto d11e7; d47c4: if (true === $hasPermissions) { goto Fa633; } goto e9ab4; eeca2: goto ddb0e; goto a3e44; c8a00: $form = $this->createReverseProxyForm(); goto a54a2; Cbac8: $hasPermissions = $this->hasPermissions(); goto d47c4; Deeaf: $response = $this->handleReverseProxyForm($request, $form); goto c163d; e9ab4: $response = $this->redirect($this->generateUrl("\143\x6c\160\137\163\151\x74\x65\x73")); goto eeca2; e13b5: return $response; goto ed4ce; da388: D0f7f: goto A6c67; ed4ce: } private function createReverseProxyForm() : Form { goto af97d; fe6ea: $form->add("\163\x75\142\x6d\151\x74", SubmitType::class, ["\x61\x74\164\x72" => ["\143\x6c\x61\x73\163" => "\x62\x74\156\40\142\164\156\x2d\x6c\x67\40\x62\x74\156\x2d\142\x6c\x75\145"], "\154\x61\x62\x65\154" => "\x43\162\x65\x61\164\145"]); goto C81e6; af97d: $form = $this->createForm("\101\x70\x70\x5c\106\x6f\x72\x6d\134\123\151\164\x65\116\x65\167\122\x65\x76\145\x72\163\x65\x50\x72\157\x78\x79\124\171\160\x65", [], ["\141\143\x74\x69\x6f\156" => $this->generateUrl("\143\x6c\x70\x5f\x73\151\x74\x65\x5f\x72\145\x76\145\x72\163\145\x5f\160\x72\157\170\171\137\156\x65\x77"), "\x6d\x65\x74\x68\x6f\x64" => "\120\117\x53\x54", "\141\164\164\162" => ["\x69\144" => "\x6e\x65\167\x2d\x72\145\x76\x65\x72\163\145\55\x70\x72\x6f\170\171\x2d\146\157\x72\155"]]); goto fe6ea; C81e6: return $form; goto a5e18; a5e18: } private function handleReverseProxyForm(Request $request, Form $form) { goto b0f4f; E2261: b2dd9: goto B69e7; aa629: $this->formErrors = $this->getErrorMessages($form); goto Bc949; Bc949: goto b2dd9; goto c36a8; b0f4f: if (true === $form->isValid()) { goto c5223; } goto aa629; C4a58: try { goto aded7; d6367: $this->createReverseProxySite($domainName, $reverseProxyUrl, $siteUser, $siteUserPassword); goto Cc178; Cc178: $eventData = ["\163\x69\x74\145" => $domainName, "\x73\x69\x74\145\x55\163\145\x72" => $siteUser, "\x72\145\166\x65\x72\x73\145\x50\x72\x6f\170\x79\125\162\154" => $reverseProxyUrl]; goto Aecbe; Cfbee: $siteUser = $form->get("\163\x69\x74\145\125\163\145\x72")->getData(); goto E9f42; faa6a: return $response; goto F6e92; Da082: F046d: goto a06b7; d0464: $vhostTemplateEntity = $this->getVhostTemplateEntity($vhostTemplateName); goto e908d; d30c0: throw new \Exception(sprintf("\x56\x68\157\163\x74\40\164\x65\x6d\x70\154\x61\164\145\40\x22\45\x73\42\40\156\x6f\164\40\146\x6f\x75\x6e\x64\56", $vhostTemplateName)); goto Fe25f; Aecbe: EventQueue::addEvent(EventQueue::EVENT_SITE_REVERSE_PROXY_CREATE, $user, $eventData, $request); goto ba2ba; ba2ba: $session->getFlashBag()->set("\x73\165\143\143\145\x73\163", $this->translator->trans("\122\145\x76\145\162\163\x65\x20\120\162\157\x78\x79\x20\x68\x61\163\x20\x62\145\145\156\40\x63\162\x65\141\164\x65\x64\56")); goto f9aba; F6e92: b2412: goto F6caa; Fe25f: goto b2412; goto Da082; f9aba: $response = $this->redirect($this->generateUrl("\143\x6c\x70\137\163\151\164\145\x73")); goto faa6a; aded7: $session = $request->getSession(); goto Effdc; d0686: $reverseProxyUrl = $form->get("\162\x65\x76\145\x72\163\x65\x50\162\157\x78\x79\x55\162\154")->getData(); goto a5390; a06b7: $user = $this->getUser(); goto d6367; Effdc: $domainName = $form->get("\x64\157\x6d\x61\151\x6e\116\x61\155\x65")->getData(); goto Cfbee; e908d: if (false === is_null($vhostTemplateEntity)) { goto F046d; } goto d30c0; a5390: $vhostTemplateName = self::REVERSE_PROXY_VHOST_TEMPLATE_NAME; goto d0464; E9f42: $siteUserPassword = $form->get("\163\x69\x74\145\125\163\x65\x72\x50\141\163\x73\x77\x6f\x72\x64")->getData(); goto d0686; F6caa: } catch (\Exception $e) { $this->logger->exception($e); $session->getFlashBag()->set("\x64\x61\156\147\145\162", $this->translator->trans("\101\156\x20\x65\x72\162\x6f\x72\x20\x68\x61\x73\x20\x6f\143\x63\x75\162\x72\145\x64\54\40\x65\162\162\x6f\162\x20\x6d\145\163\x73\x61\x67\x65\x3a\x20\x25\x65\162\x72\x6f\162\x4d\145\x73\163\x61\147\x65\x25", ["\45\145\x72\162\157\x72\115\145\163\x73\x61\x67\145\45" => $e->getMessage()])); } goto E2261; c36a8: c5223: goto C4a58; B69e7: } public function newNodejsSite(Request $request) : Response { goto A394e; f2e7a: $user = $this->getUser(); goto A98e6; A98e6: $nodejsSettings = new NodejsSettingsEntity(); goto ad878; F1611: $response = $this->redirect($this->generateUrl("\143\x6c\x70\x5f\163\x69\164\x65\163")); goto b3a32; ad878: $form = $this->createNodejsSiteForm($nodejsSettings); goto e727d; cd953: if (!(false === is_null($response))) { goto f2286; } goto ee5e6; b9e70: if (true === $hasPermissions) { goto e4ae0; } goto F1611; A5266: $response = $this->handleCreateNodejsSiteForm($request, $form); goto cd953; Dedca: f2286: goto Aacf6; A394e: $hasPermissions = $this->hasPermissions(); goto b9e70; b2a2a: if (!(true === $form->isSubmitted())) { goto e47eb; } goto A5266; B0ed4: return $response; goto Dda22; b3a32: goto dcba5; goto eb50a; d5d49: d21c0: goto A4319; ccacb: dcba5: goto B0ed4; Aacf6: e47eb: goto d5d49; ee5e6: return $response; goto Dedca; e727d: if (!(true === $request->isMethod("\x50\x4f\123\x54"))) { goto d21c0; } goto Fdde1; A4319: $response = $this->render("\x46\x72\x6f\156\164\x65\x6e\144\57\123\151\x74\145\57\x4e\x65\167\x2f\156\157\x64\x65\x6a\x73\x2e\x68\164\155\x6c\56\164\x77\151\147", ["\165\163\x65\162" => $user, "\146\157\162\x6d" => $form->createView(), "\x66\x6f\x72\155\x45\x72\x72\157\162\x73" => $this->formErrors]); goto ccacb; eb50a: e4ae0: goto f2e7a; Fdde1: $form->handleRequest($request); goto b2a2a; Dda22: } private function createNodejsSiteForm(NodejsSettingsEntity $nodejsSettings) : Form { goto D7b85; B9bd7: $form->add("\163\x75\x62\155\151\x74", SubmitType::class, ["\x61\x74\164\162" => ["\143\154\141\163\163" => "\x62\164\156\40\142\164\156\x2d\154\147\40\142\164\156\55\142\154\165\145"], "\x6c\141\x62\145\x6c" => "\103\x72\x65\x61\x74\145"]); goto A4db2; D7b85: $form = $this->createForm("\101\x70\160\134\106\157\x72\155\134\x53\151\x74\145\x4e\145\x77\x4e\157\144\x65\x6a\163\124\171\160\145", $nodejsSettings, ["\x61\x63\164\151\x6f\156" => $this->generateUrl("\143\154\160\137\163\151\164\x65\137\x6e\x6f\144\145\x6a\163\137\156\145\x77"), "\155\145\x74\x68\157\x64" => "\x50\117\x53\x54", "\141\x74\x74\162" => ["\151\x64" => "\156\x65\x77\x2d\156\157\144\145\152\163\55\x73\151\164\x65\x2d\x66\157\162\x6d"]]); goto B9bd7; A4db2: return $form; goto e6c79; e6c79: } private function handleCreateNodejsSiteForm(Request $request, Form $form) { goto f8393; B6ca5: $this->formErrors = $this->getErrorMessages($form); goto a8c58; D3e3e: E35a7: goto E0b51; B7389: e414c: goto c6267; c6267: try { goto f94e4; b53fb: if (false === is_null($vhostTemplateEntity)) { goto Af001; } goto f843f; c510a: Af001: goto A9a74; e542e: $port = $form->get("\x70\157\x72\x74")->getData(); goto Cf0b2; b13ee: return $response; goto bd71c; C1bbc: $eventData = ["\x73\151\x74\145" => $domainName, "\163\x69\164\x65\x55\x73\145\x72" => $siteUser, "\x6e\x6f\x64\x65\x6a\x73\x56\145\162\x73\151\x6f\x6e" => $nodejsVersion, "\x70\x6f\x72\164" => $port]; goto b45ed; Dd3c9: $vhostTemplateName = self::NODEJS_VHOST_TEMPLATE_NAME; goto cdb3c; B8d3f: $nodejsVersion = $form->get("\156\x6f\x64\145\x6a\x73\126\x65\x72\163\151\x6f\156")->getData(); goto e542e; cdb3c: $vhostTemplateEntity = $this->getVhostTemplateEntity($vhostTemplateName); goto b53fb; b0465: $response = $this->redirect($this->generateUrl("\x63\154\x70\x5f\x73\x69\164\145\x73")); goto b13ee; ba8cd: $this->createNodejsSite($domainName, $nodejsVersion, $port, $siteUser, $siteUserPassword); goto C1bbc; a2001: goto Ef6fb; goto c510a; f94e4: $session = $request->getSession(); goto A5a3d; b5caa: $siteUserPassword = $form->get("\163\151\x74\145\x55\163\145\x72\120\x61\x73\163\x77\157\x72\x64")->getData(); goto Dd3c9; bd71c: Ef6fb: goto A4f91; A456a: $session->getFlashBag()->set("\x73\165\x63\x63\x65\x73\x73", $this->translator->trans("\x53\x69\164\145\40\x68\x61\x73\40\x62\x65\x65\156\x20\143\x72\x65\x61\x74\x65\x64\56")); goto b0465; Cf0b2: $siteUser = $form->get("\x73\151\164\x65\x55\x73\145\x72")->getData(); goto b5caa; f843f: throw new \Exception(sprintf("\x56\150\x6f\x73\164\x20\164\x65\x6d\160\154\141\x74\x65\40\42\45\163\42\x20\x6e\157\x74\x20\x66\157\x75\156\x64\56", $vhostTemplateName)); goto a2001; b45ed: EventQueue::addEvent(EventQueue::EVENT_SITE_NODEJS_CREATE, $user, $eventData, $request); goto A456a; A9a74: $user = $this->getUser(); goto ba8cd; A5a3d: $domainName = $form->get("\144\157\x6d\x61\x69\x6e\116\x61\x6d\x65")->getData(); goto B8d3f; A4f91: } catch (\Exception $e) { $this->logger->exception($e); $session->getFlashBag()->set("\x64\141\156\147\145\x72", $this->translator->trans("\x41\x6e\40\x65\x72\162\x6f\162\40\150\141\x73\x20\157\x63\x63\x75\x72\x72\145\144\54\x20\x65\162\162\x6f\x72\40\155\x65\x73\x73\141\x67\145\72\x20\x25\145\x72\x72\x6f\x72\115\x65\x73\x73\141\147\145\45", ["\x25\x65\x72\x72\x6f\162\x4d\145\163\x73\141\x67\x65\45" => $e->getMessage()])); } goto D3e3e; a8c58: goto E35a7; goto B7389; f8393: if (true === $form->isValid()) { goto e414c; } goto B6ca5; E0b51: } public function newStaticSite(Request $request) : Response { goto E1f42; Da6c4: if (!(true === $form->isSubmitted())) { goto d0d90; } goto daa63; e152a: $response = $this->render("\106\x72\157\156\164\x65\156\x64\57\123\151\164\145\57\x4e\x65\167\x2f\163\164\x61\164\151\x63\56\x68\x74\x6d\x6c\x2e\164\x77\x69\147", ["\165\x73\x65\162" => $user, "\146\157\162\155" => $form->createView(), "\146\x6f\162\x6d\105\x72\x72\x6f\162\163" => $this->formErrors]); goto f65bf; Fd04e: if (!(false === is_null($response))) { goto Aedb6; } goto C586a; C586a: return $response; goto d2e7b; daa63: $response = $this->handleCreateStaticSiteForm($request, $form); goto Fd04e; a13ec: $response = $this->redirect($this->generateUrl("\143\154\160\x5f\x73\x69\164\145\163")); goto E312d; C4599: $form->handleRequest($request); goto Da6c4; B8a22: return $response; goto f603c; D77bf: $form = $this->createStaticSiteForm(); goto B6721; dfbe6: d3d39: goto e152a; E1f42: $hasPermissions = $this->hasPermissions(); goto ceed6; De372: $user = $this->getUser(); goto D77bf; ceed6: if (true === $hasPermissions) { goto a916b; } goto a13ec; d2e7b: Aedb6: goto f3899; f65bf: Ccaa2: goto B8a22; c7270: a916b: goto De372; f3899: d0d90: goto dfbe6; E312d: goto Ccaa2; goto c7270; B6721: if (!(true === $request->isMethod("\x50\117\x53\x54"))) { goto d3d39; } goto C4599; f603c: } private function createStaticSiteForm() : Form { goto e3578; D8bec: $form->add("\x73\x75\142\x6d\x69\x74", SubmitType::class, ["\141\164\x74\x72" => ["\143\x6c\x61\163\163" => "\142\164\x6e\x20\142\164\x6e\x2d\154\147\40\142\x74\156\x2d\x62\154\x75\x65"], "\154\x61\x62\145\x6c" => "\x43\x72\145\x61\x74\x65"]); goto D4d98; e3578: $form = $this->createForm("\x41\160\160\x5c\x46\157\162\155\134\x53\151\164\x65\116\145\167\123\x74\x61\x74\x69\143\x54\x79\x70\145", [], ["\141\143\164\151\x6f\156" => $this->generateUrl("\143\x6c\x70\137\x73\151\x74\145\137\163\x74\x61\164\151\x63\x5f\156\145\x77"), "\x6d\145\164\150\157\x64" => "\x50\117\x53\124", "\141\164\x74\162" => []]); goto D8bec; D4d98: return $form; goto C1c48; C1c48: } private function handleCreateStaticSiteForm(Request $request, Form $form) { goto f28a9; f28a9: if (true === $form->isValid()) { goto D45f5; } goto a7292; c9f64: goto c37bd; goto Cc11e; D1d2e: c37bd: goto b9a96; e002c: try { goto C54bd; Ca7bb: dea91: goto c51a8; Acae3: EventQueue::addEvent(EventQueue::EVENT_SITE_STATIC_CREATE, $user, $eventData, $request); goto Dd42c; Dd42c: $session->getFlashBag()->set("\x73\165\x63\143\x65\x73\163", $this->translator->trans("\x53\151\x74\x65\40\x68\x61\163\x20\142\145\x65\156\x20\x63\162\x65\141\164\x65\x64\x2e")); goto d17ff; ad510: goto dea91; goto b5808; d6e6a: $vhostTemplateName = self::STATIC_SITE_VHOST_TEMPLATE_NAME; goto c96ec; b5808: D39b2: goto C329e; D5abb: $domainName = $form->get("\x64\157\155\141\151\x6e\116\x61\x6d\x65")->getData(); goto B8602; d17ff: $response = $this->redirect($this->generateUrl("\143\154\160\x5f\163\x69\x74\x65\163")); goto Fc293; d4106: if (false === is_null($vhostTemplateEntity)) { goto D39b2; } goto Fdb6f; c96ec: $vhostTemplateEntity = $this->getVhostTemplateEntity($vhostTemplateName); goto d4106; C329e: $user = $this->getUser(); goto bf9ea; Fdb6f: throw new \Exception(sprintf("\126\150\x6f\163\x74\40\164\145\x6d\160\x6c\141\x74\x65\x20\42\45\x73\x22\x20\156\157\164\40\x66\157\165\156\144\56", $vhostTemplateName)); goto ad510; C54bd: $session = $request->getSession(); goto D5abb; B8602: $siteUser = $form->get("\x73\x69\164\145\x55\163\x65\x72")->getData(); goto Bac11; Bac11: $siteUserPassword = $form->get("\x73\151\164\x65\125\163\145\162\x50\141\163\x73\x77\157\162\x64")->getData(); goto d6e6a; Fc293: return $response; goto Ca7bb; bf9ea: $this->createStaticSite($domainName, $siteUser, $siteUserPassword); goto F1fa2; F1fa2: $eventData = ["\163\x69\164\145" => $domainName, "\163\x69\164\x65\x55\x73\145\x72" => $siteUser]; goto Acae3; c51a8: } catch (\Exception $e) { $this->logger->exception($e); $session->getFlashBag()->set("\144\141\x6e\x67\x65\x72", $this->translator->trans("\101\x6e\x20\x65\162\x72\x6f\162\x20\x68\141\163\x20\157\x63\x63\165\x72\x72\x65\144\54\40\145\162\x72\x6f\162\40\155\x65\163\163\x61\x67\145\72\40\x25\145\x72\162\157\x72\115\x65\x73\163\141\147\x65\x25", ["\x25\145\x72\x72\x6f\162\x4d\x65\x73\163\141\x67\x65\45" => $e->getMessage()])); } goto D1d2e; a7292: $this->formErrors = $this->getErrorMessages($form); goto c9f64; Cc11e: D45f5: goto e002c; b9a96: } public function newPhpSite(Request $request) : Response { goto d21af; f3e30: if (true === $hasPermissions) { goto d25f4; } goto E5502; df9cc: d25f4: goto c3b77; De87c: return $response; goto C1673; E4140: Aa5b2: goto eac65; e2dbd: Ca911: goto ceaa7; ceaa7: B0b1c: goto F2a32; c81cd: goto Aa5b2; goto df9cc; db285: if (!(true === $request->isMethod("\120\117\123\x54"))) { goto B0b1c; } goto ed709; eac65: return $response; goto d8ced; d21af: $hasPermissions = $this->hasPermissions(); goto f3e30; E5502: $response = $this->redirect($this->generateUrl("\143\154\160\137\x73\x69\164\145\x73")); goto c81cd; F2a32: $response = $this->render("\106\x72\157\156\164\x65\x6e\144\57\x53\x69\x74\x65\57\116\x65\167\57\x70\x68\160\56\x68\164\155\154\56\164\x77\x69\x67", ["\x75\163\x65\x72" => $user, "\x66\x6f\162\155" => $form->createView(), "\x66\x6f\x72\x6d\105\x72\x72\157\162\x73" => $this->formErrors]); goto E4140; a0ff9: $response = $this->handleCreatePhpSiteForm($request, $form); goto Dd326; c3b77: $user = $this->getUser(); goto cc441; Dd326: if (!(false === is_null($response))) { goto d65ad; } goto De87c; Ad920: if (!(true === $form->isSubmitted())) { goto Ca911; } goto a0ff9; cc441: $form = $this->createPhpSiteForm(); goto db285; C1673: d65ad: goto e2dbd; ed709: $form->handleRequest($request); goto Ad920; d8ced: } private function createPhpSiteForm() : Form { goto F7034; Ccfe3: $form->add("\x73\x75\x62\155\151\164", SubmitType::class, ["\x61\164\164\x72" => ["\x63\x6c\x61\163\163" => "\x62\x74\x6e\x20\142\164\156\55\x6c\x67\40\142\164\156\x2d\142\x6c\x75\x65"], "\x6c\141\x62\145\154" => "\103\x72\145\141\x74\145"]); goto Add3d; F7034: $form = $this->createForm("\x41\x70\160\134\106\x6f\x72\x6d\x5c\123\151\164\x65\x4e\x65\x77\x50\150\160\x54\x79\160\145", [], ["\141\x63\x74\x69\157\156" => $this->generateUrl("\x63\154\x70\x5f\x73\151\164\x65\x5f\160\x68\x70\137\156\145\x77"), "\155\145\x74\x68\x6f\x64" => "\x50\117\x53\124", "\x61\164\x74\x72" => []]); goto Ccfe3; Add3d: return $form; goto d3024; d3024: } private function handleCreatePhpSiteForm(Request $request, Form $form) { goto Dcb96; C85d7: try { goto ef997; Fa6b3: c9cd5: goto b5094; Bfbaa: $user = $this->getUser(); goto Ba8de; Bc012: $domainName = $form->get("\144\157\x6d\x61\151\x6e\116\141\155\145")->getData(); goto df667; A36d8: if (false === is_null($vhostTemplateEntity)) { goto cfa6a; } goto E95d0; a9638: cfa6a: goto Bfbaa; E175b: $varnishCacheSettings = (array) json_decode($vhostTemplateEntity->getVarnishCacheSettings(), true); goto a90f2; bd645: $vhostTemplateEntity = $this->getVhostTemplateEntity($vhostTemplateName); goto A36d8; Ba8de: $rootDirectory = $domainName; goto c522d; A2e64: $siteUser = $form->get("\x73\x69\x74\x65\x55\x73\145\162")->getData(); goto Fcd40; a73de: $this->createPhpSite($domainName, $rootDirectory, $phpVersion, $vhostTemplateName, $siteUser, $siteUserPassword, $varnishCacheSettings); goto cb2ae; Fcd40: $siteUserPassword = $form->get("\x73\x69\164\x65\125\163\x65\162\x50\x61\x73\163\x77\157\162\x64")->getData(); goto bd645; D29d7: return $response; goto Fa6b3; A4c07: f21e6: goto e4bba; D55f6: goto c9cd5; goto a9638; e3100: if (!(false === empty($vhostTemplateEntity->getVarnishCacheSettings()))) { goto a0725; } goto E175b; a90f2: a0725: goto a73de; E95d0: throw new \Exception(sprintf("\126\150\157\163\164\x20\x74\145\155\x70\x6c\141\164\x65\x20\42\45\163\x22\x20\x6e\157\x74\40\x66\x6f\x75\x6e\144\x2e", $vhostTemplateName)); goto D55f6; ef997: $session = $request->getSession(); goto db1b4; db1b4: $vhostTemplateName = $form->get("\x61\160\x70\154\151\143\x61\164\x69\157\x6e")->getData(); goto Bc012; ca823: EventQueue::addEvent(EventQueue::EVENT_SITE_PHP_CREATE, $user, $eventData, $request); goto e4a45; c522d: if (!(false === empty($vhostTemplateEntity->getRootDirectory()))) { goto f21e6; } goto Cb9c6; Ec65b: $response = $this->redirect($this->generateUrl("\143\x6c\x70\137\163\x69\164\x65\163")); goto D29d7; df667: $phpVersion = $form->get("\160\150\x70\x56\x65\x72\x73\151\x6f\156")->getData(); goto A2e64; e4a45: $session->getFlashBag()->set("\163\x75\x63\x63\145\x73\163", $this->translator->trans("\123\x69\164\x65\x20\150\x61\163\x20\142\x65\145\x6e\40\143\x72\145\141\x74\x65\x64\x2e")); goto Ec65b; e4bba: $varnishCacheSettings = []; goto e3100; Cb9c6: $rootDirectory = sprintf("\45\x73\x2f\x25\x73", $rootDirectory, ltrim(rtrim($vhostTemplateEntity->getRootDirectory(), "\57"), "\57")); goto A4c07; cb2ae: $eventData = ["\163\151\x74\x65" => $domainName, "\x76\x68\x6f\163\164\x54\x65\x6d\x70\x6c\141\x74\145\x4e\x61\x6d\145" => $vhostTemplateName, "\x70\x68\x70\x56\x65\162\x73\151\x6f\156" => $phpVersion, "\x73\x69\164\x65\125\163\145\162" => $siteUser]; goto ca823; b5094: } catch (\Exception $e) { $this->logger->exception($e); $session->getFlashBag()->set("\x64\x61\156\147\145\x72", $this->translator->trans("\101\x6e\x20\x65\162\x72\157\162\x20\x68\x61\x73\40\157\143\x63\x75\162\162\x65\x64\x2c\40\145\162\x72\157\162\40\x6d\x65\x73\x73\141\x67\x65\x3a\40\x25\145\162\x72\x6f\x72\x4d\145\163\x73\141\147\145\45", ["\45\x65\x72\x72\x6f\162\115\x65\163\x73\x61\147\145\45" => $e->getMessage()])); } goto Ce9ef; f9fc8: goto e222f; goto Ef27b; Ce9ef: e222f: goto a3e5b; Ef27b: F289c: goto C85d7; bd88d: $this->formErrors = $this->getErrorMessages($form); goto f9fc8; Dcb96: if (true === $form->isValid()) { goto F289c; } goto bd88d; a3e5b: } public function newWordPressSite(Request $request) : Response { goto D23d7; d3267: if (true === $hasPermissions) { goto cc3d9; } goto f34c9; Ae2f8: $user = $this->getUser(); goto Da783; Df7bc: if (!(true === $request->isMethod("\120\x4f\123\124"))) { goto Da3e0; } goto a9569; ddee2: goto a1e53; goto De785; f34c9: $response = $this->redirect($this->generateUrl("\143\x6c\160\x5f\x73\x69\164\145\x73")); goto ddee2; De785: cc3d9: goto Ae2f8; a9569: $form->handleRequest($request); goto Bbbbb; e7616: a1e53: goto F4d33; Cbcb3: $response = $this->handleCreateWordPressSiteForm($request, $form); goto D8b9a; D8b9a: if (!(false === is_null($response))) { goto C68cf; } goto e3e04; C984f: Da3e0: goto F8cfc; Da783: $form = $this->createWordPressSiteForm(); goto Df7bc; e3e04: return $response; goto A495f; D23d7: $hasPermissions = $this->hasPermissions(); goto d3267; F8cfc: $response = $this->render("\106\x72\x6f\x6e\164\145\156\144\x2f\123\151\x74\x65\57\x4e\145\167\57\167\x6f\x72\144\x70\x72\145\x73\163\56\x68\164\x6d\x6c\56\164\167\x69\147", ["\165\163\x65\162" => $user, "\x66\157\162\x6d" => $form->createView(), "\x66\157\x72\x6d\x45\162\162\x6f\x72\x73" => $this->formErrors]); goto e7616; b908f: fb04b: goto C984f; Bbbbb: if (!(true === $form->isSubmitted())) { goto fb04b; } goto Cbcb3; F4d33: return $response; goto e26d2; A495f: C68cf: goto b908f; e26d2: } private function createWordPressSiteForm() : Form { goto d723c; B4835: $form->add("\163\165\x62\155\151\x74", SubmitType::class, ["\141\x74\164\x72" => ["\143\154\141\x73\x73" => "\x62\164\x6e\x20\142\x74\x6e\x2d\154\147\x20\142\164\156\55\x62\x6c\x75\x65"], "\154\141\142\x65\154" => "\103\x72\145\141\x74\x65"]); goto bb1db; d723c: $form = $this->createForm("\101\x70\x70\134\106\x6f\x72\155\134\x53\x69\x74\145\x4e\x65\x77\x57\157\162\144\x50\162\145\163\x73\x54\x79\x70\145", [], ["\x61\143\x74\x69\157\x6e" => $this->generateUrl("\143\x6c\x70\x5f\x73\x69\x74\145\137\167\157\162\144\x70\x72\145\163\x73\137\x6e\145\x77"), "\x6d\x65\164\x68\x6f\144" => "\x50\x4f\x53\x54", "\x61\x74\164\x72" => ["\x69\144" => "\x6e\x65\x77\55\x77\x6f\162\x64\x70\162\x65\163\163\x2d\x73\x69\x74\145\55\146\x6f\162\155"]]); goto B4835; bb1db: return $form; goto daff2; daff2: } private function handleCreateWordPressSiteForm(Request $request, Form $form) { goto B2a19; d617c: goto dbc25; goto Af7d5; B250b: try { goto e51f8; Eb301: $this->siteEntityManager->updateEntity($siteEntity); goto F7f8d; d8258: $databaseEntity->setDatabaseServer($activeDatabaseServerEntity); goto D88d6; F1b03: $varnishCacheSettings = (array) json_decode($vhostTemplateEntity->getVarnishCacheSettings(), true); goto d4fa3; bff2a: $databaseEntity->setSite($siteEntity); goto c587a; a1080: $siteUserPassword = $form->get("\163\x69\x74\145\x55\163\x65\x72\x50\141\x73\163\167\x6f\162\x64")->getData(); goto B1e4a; D5a27: $eventData = ["\x73\x69\x74\x65" => $domainName, "\163\x69\x74\x65\x55\x73\145\x72" => $siteUser]; goto A19a4; fe830: $phpVersion = $vhostTemplateEntity->getPhpVersion(); goto c1328; b28bb: $siteEntity->setApplication($vhostTemplateEntity->getName()); goto e2c0e; A775a: $databaseUserName = $this->getUniqueDatabaseUserName($generatedName); goto B1e12; D6ccc: $adminEmail = $form->get("\141\x64\155\151\x6e\105\155\x61\x69\154")->getData(); goto E6bd5; bf4d9: $databaseUserEntity->setUserName($databaseUserName); goto f82b3; Fe931: $title = $form->get("\x73\151\x74\x65\124\151\x74\x6c\x65")->getData(); goto df0d4; E3ecc: $databaseManager->createUser($databaseUserEntity); goto Eb301; F913f: if (!(false === empty($vhostTemplateEntity->getVarnishCacheSettings()))) { goto b363c; } goto F1b03; D88d6: $databaseEntity->setName($databaseName); goto bff2a; E5926: $session->set("\x73\151\x74\145\x43\162\x65\144\145\x6e\x74\151\x61\154\163", $siteCredentials); goto D5a27; F18eb: A329e: goto E5926; F7f8d: $this->installWordPress($isMultiSite, $url, $title, $adminUserName, $adminPassword, $adminEmail, $siteEntity, $activeDatabaseServerEntity, $databaseEntity, $databaseUserEntity); goto a182f; Cff75: $databaseUserEntity->setPermissions(DatabaseUserEntity::PERMISSIONS_READ_WRITE); goto A7f4e; c1328: $varnishCacheSettings = []; goto F913f; B1e12: $databaseUserPassword = PasswordGenerator::generate(self::PASSWORD_LENGTH); goto A6d44; d10b9: $response = $this->redirect($this->generateUrl("\x63\x6c\160\137\x73\151\164\x65\137\x77\157\162\144\160\162\x65\163\163\137\151\156\163\164\x61\154\x6c\x65\144")); goto A6610; A6d44: $url = sprintf("\x68\x74\x74\x70\x73\x3a\57\x2f\45\x73", $siteEntity->getDomainName()); goto Fe931; A6610: return $response; goto ac810; B729e: $siteEntity->addDatabase($databaseEntity); goto c3fdf; d11dd: $activeDatabaseServerEntity = $this->databaseServerEntityManager->getActiveDatabaseServer(); goto F3e69; A19a4: EventQueue::addEvent(EventQueue::EVENT_SITE_WORDPRESS_CREATE, $user, $eventData, $request); goto d10b9; fd841: $domainName = $form->get("\144\x6f\155\x61\x69\156\116\x61\155\x65")->getData(); goto Ffd1f; bb8d8: $siteCredentials = [$this->translator->trans("\123\x69\x74\145") => [$this->translator->trans("\111\x50\x20\x41\x64\x64\162\145\163\x73") => $instance->getIpv4PublicIp(), $this->translator->trans("\104\x6f\x6d\x61\151\156\x20\116\141\155\x65") => sprintf("\x68\x74\164\160\x73\72\x2f\57\x25\x73", $domainName), $this->translator->trans("\x53\x69\164\145\40\125\x73\x65\162") => $siteUser, $this->translator->trans("\120\141\163\163\x77\157\162\144") => $siteUserPassword]]; goto d4aa5; e51f8: $user = $this->getUser(); goto C6102; Fd9bc: $databaseName = $this->getUniqueDatabaseName($generatedName); goto A775a; Ce89e: $siteCredentials["\x57\157\x72\x64\120\x72\x65\163\163"] = [$this->translator->trans("\x41\x64\155\x69\156\x20\x45\x2d\x4d\x61\151\x6c") => $adminEmail, $this->translator->trans("\101\x64\x6d\151\156\40\125\163\x65\x72\x20\x4e\x61\155\x65") => $adminUserName, $this->translator->trans("\x41\144\155\151\156\40\x50\141\x73\x73\167\x6f\x72\144") => $adminPassword, $this->translator->trans("\101\144\x6d\151\156\x20\125\162\154") => sprintf("\x68\164\x74\160\x73\x3a\57\x2f\x25\x73\x2f\167\x70\55\x61\144\x6d\151\x6e\57", rtrim($domainName, "\57"))]; goto F18eb; a182f: $siteCredentials[$this->translator->trans("\104\x61\x74\x61\x62\141\163\145")] = [$this->translator->trans("\x48\x6f\163\164") => $activeDatabaseServerEntity->getHost(), $this->translator->trans("\120\157\162\x74") => $activeDatabaseServerEntity->getPort(), $this->translator->trans("\104\141\164\x61\x62\x61\163\145\x20\116\x61\155\x65") => $databaseName, $this->translator->trans("\104\141\x74\141\142\141\163\x65\x20\x55\x73\145\162\x20\116\141\155\145") => $databaseUserName, $this->translator->trans("\104\x61\x74\141\x62\141\x73\x65\40\125\x73\145\x72\40\x50\141\163\x73\x77\x6f\x72\144") => $databaseUserPassword]; goto Ce89e; Ffd1f: $generatedName = $this->generateNameFromDomainName($domainName); goto F3d10; f82b3: $databaseUserEntity->setPassword(Crypto::encrypt($databaseUserPassword)); goto Cff75; c3fdf: $databaseManager = new DatabaseManager($activeDatabaseServerEntity); goto f6226; bb220: $adminPassword = $form->get("\x61\x64\155\151\x6e\120\x61\163\163\x77\157\x72\144")->getData(); goto D6ccc; df0d4: $adminUserName = $form->get("\141\144\x6d\151\156\x55\x73\x65\x72\116\x61\155\145")->getData(); goto bb220; d4fa3: b363c: goto db492; d4aa5: if (!(false === is_null($siteEntity))) { goto A329e; } goto Fd9bc; db492: $siteEntity = $this->createPhpSite($domainName, $domainName, $phpVersion, self::WORDPRESS_VHOST_TEMPLATE_NAME, $siteUser, $siteUserPassword, $varnishCacheSettings); goto b28bb; C6102: $session = $request->getSession(); goto fd841; F3d10: $siteUser = $form->get("\x73\151\164\145\x55\x73\145\x72")->getData(); goto a1080; c587a: $databaseUserEntity = $this->databaseUserEntityManager->createEntity(); goto bf4d9; cf87f: $databaseEntity->addUser($databaseUserEntity); goto B729e; B1e4a: $vhostTemplateEntity = $this->getVhostTemplateEntity(self::WORDPRESS_VHOST_TEMPLATE_NAME); goto fe830; F3e69: $databaseEntity = $this->databaseEntityManager->createEntity(); goto d8258; E6bd5: $isMultiSite = $form->get("\155\165\154\164\x69\123\x69\x74\x65")->getData(); goto d11dd; f6226: $databaseManager->createDatabase($databaseEntity); goto E3ecc; A7f4e: $databaseUserEntity->setDatabase($databaseEntity); goto cf87f; e2c0e: $instance = $request->attributes->get("\x69\156\x73\x74\x61\x6e\x63\x65"); goto bb8d8; ac810: } catch (\Exception $e) { $this->logger->exception($e); $session->getFlashBag()->set("\x64\x61\x6e\x67\x65\x72", $this->translator->trans("\101\x6e\40\145\162\162\157\162\x20\x68\141\163\x20\157\x63\143\165\x72\x72\145\x64\54\x20\x65\x72\x72\x6f\162\x20\x6d\145\x73\x73\141\147\x65\72\40\x25\x65\162\x72\x6f\162\x4d\x65\x73\163\141\147\x65\45", ["\x25\x65\x72\162\157\x72\115\145\163\163\141\147\145\x25" => $e->getMessage()])); } goto cfd56; B1355: $this->formErrors = $this->getErrorMessages($form); goto d617c; Af7d5: cb497: goto B250b; B2a19: if (true === $form->isValid()) { goto cb497; } goto B1355; cfd56: dbc25: goto b7d7a; b7d7a: } private function installWordPress(bool $isMultiSite, string $url, string $title, string $adminUserName, string $adminPassword, string $adminEmail, SiteEntity $siteEntity, DatabaseServerEntity $databaseServerEntity, DatabaseEntity $databaseEntity, DatabaseUserEntity $databaseUserEntity) : void { goto c9fab; f211a: $this->wordPressMultiSiteConfigValues["\x44\117\x4d\101\111\x4e\x5f\x43\x55\122\122\x45\x4e\124\x5f\x53\111\x54\x45"]["\166\141\x6c\x75\x65"] = $siteEntity->getDomainName(); goto a6445; fb817: $databaseUserPassword = Crypto::decrypt($databaseUserEntity->getPassword()); goto E0121; fbdc1: $wordPressInstaller->createConfig($databaseHost, $databaseName, $databaseUserName, $databaseUserPassword); goto F0a04; B53cd: cd643: goto Af623; Ff9ec: $wordPressInstaller->resetPermissions(); goto Cdba6; e82d2: $wordPressInstaller->downloadAndExtractLatestVersion(); goto fbdc1; f2594: foreach ($this->wordPressDefaultConfigValues as $key => $config) { $wordPressInstaller->setConfigValue($key, $config["\166\141\x6c\165\x65"], $config["\x72\141\167"]); c1ed9: } goto B53cd; Af623: if (!(true === $isMultiSite)) { goto a0756; } goto f211a; Bf4b8: $databaseName = $databaseEntity->getName(); goto Bfdb8; F0a04: $wordPressInstaller->installCore($isMultiSite, $url, $title, $adminUserName, $adminPassword, $adminEmail); goto f2594; a0b27: E5365: goto Ff432; c9fab: $databaseHost = sprintf("\x25\163\x3a\45\x73", $databaseServerEntity->getHost(), $databaseServerEntity->getPort()); goto Bf4b8; E0121: $wordPressInstaller = new WordPressInstaller($siteEntity); goto e82d2; a6445: foreach ($this->wordPressMultiSiteConfigValues as $key => $config) { $wordPressInstaller->setConfigValue($key, $config["\x76\x61\154\x75\x65"], $config["\x72\141\167"]); B11a3: } goto a0b27; Ff432: a0756: goto Ff9ec; Bfdb8: $databaseUserName = $databaseUserEntity->getUserName(); goto fb817; Cdba6: } private function createPhpSite(string $domainName, string $rootDirectory, string $phpVersion, string $vhostTemplateName, string $siteUser, string $siteUserPassword, array $varnishCacheSettings = []) : SiteEntity { goto acbee; d5ced: goto ccf46; goto Da6f4; D6f3a: $phpSiteCreator->reloadNginxService(); goto a157a; B1043: $certificateEntity = $this->certificateEntityManager->createEntity(); goto c4a39; cf10a: if (!(count($siteConstraints) || count($phpSettingsConstraints))) { goto f5ac7; } goto Fe64d; f476e: if (true === isset($_ENV["\x41\x50\x50\137\x48\x54\x54\120\63"]) && "\164\x72\165\145" === $_ENV["\x41\x50\120\x5f\110\x54\124\120\x33"]) { goto F1762; } goto e0fa3; b769e: $rsaKeyGenerator = new RsaKeyGenerator(); goto B689f; a96a9: $siteEntity->setVhostTemplate($vhostTemplate); goto a581b; Fd75c: $phpSite->setVarnishCache(true); goto Fb09a; Bd1b3: $subjectAlternativeNames[] = $registrableDomain; goto d7d3a; Ddb72: fd989: goto e193e; Da0a6: $vhostTemplate->addProcessor(new RedirectServerNameProcessor()); goto de6b1; ebc0c: $varnishCacheSettings = array_merge($defaultVarnishCacheSettings, $varnishCacheSettings); goto Acc29; e699b: $siteEntity->setVarnishCache(true); goto Ddb72; cb304: $siteEntity->setRootDirectory($rootDirectory); goto Dcbe6; Fe64d: $constraintViolationList = new ConstraintViolationList(); goto Fe509; D1112: $certificateEntity->setCertificate($selfSignedCertificate); goto B71f4; f45f6: $phpSite->setVarnishCacheSettings($varnishCacheSettings); goto ce7e2; F4b66: $phpSiteCreator->createIndexPhp(); goto d9033; A2a44: $resolvedDomainName = $this->domainNameParser->resolveDomainName($domainName); goto Bf590; a7d52: $phpSiteCreator->createUser(); goto B9f43; acbee: $varnishCache = false === empty($varnishCacheSettings) ? true : false; goto ed045; F918a: $constraintViolationList->addAll($phpSettingsConstraints); goto d984e; C9f65: $phpSite->setUser($siteEntity->getUser()); goto cf170; ede39: $siteEntity->setUser($siteUser); goto c981b; d79ae: Dd1d8: goto f1ac0; Df7db: $certificateEntity->setPrivateKey($privateKey->getPEM()); goto D1112; Fbbbf: $phpSettingsEntity->setPhpVersion($phpVersion); goto f1796; Cbfc3: if (!(false === empty($redirectionVhostTemplate))) { goto Dd1d8; } goto C97c1; cc18e: $vhostTemplate->addProcessor(new ServerNameProcessor()); goto Da0a6; a581b: $phpSettingsEntity = new PhpSettingsEntity(); goto A2ce8; C27e6: $siteEntity->setType(SiteEntity::TYPE_PHP); goto Dca40; c0065: $vhostTemplate->resetProcessors(); goto cc18e; Dca40: $siteEntity->setDomainName($domainName); goto ede39; f1796: $phpSettingsEntity->setSite($siteEntity); goto d1936; ed045: $rootDirectory = trim(rtrim(ltrim($rootDirectory, "\x2f")), "\57"); goto A2a44; ecae8: $certificateEntity->setDefaultCertificate(true); goto Ab39e; Dfdcf: $phpSite->setPhpSettings($phpSettingsEntity); goto e3fda; Da6f4: F1762: goto dbadb; A3d74: $phpSite->setDomainName($domainName); goto d5036; B0fe3: $subjectAlternativeNames[] = sprintf("\x77\167\x77\56\45\x73", $domainName); goto E37f3; A648d: $vhostTemplate->setContent($siteEntity->getVhostTemplate()); goto c0065; C97c1: $vhostTemplate = sprintf("\x25\163\45\163", $redirectionVhostTemplate, $vhostTemplate); goto d79ae; dbadb: $redirectionVhostTemplateFile = realpath(dirname(__FILE__) . "\x2f\x2e\56\x2f\x2e\56\x2f\56\x2e\x2f\162\145\163\x6f\x75\162\x63\145\x73\57\156\147\151\x6e\170\x2f\x76\150\x6f\x73\x74\x5f\164\145\155\160\154\141\x74\145\x2f\162\145\144\x69\162\145\143\164\55\150\x74\164\160\63"); goto f43f7; A63da: $certificateEntity->setCsr($csr); goto Df7db; d5036: $phpSite->setRegistrableDomain($registrableDomain); goto a6fc6; a0c69: f5ac7: goto B1043; Abee9: $csrGenerator = new CsrGenerator($privateKey, $distinguishedName); goto B3b79; f94e8: $vhostTemplateEntity = $this->getVhostTemplateEntity($vhostTemplateName); goto Acf59; a6fc6: $phpSite->setSubdomain($subdomain); goto fe4bf; Acf59: $vhostTemplate = $vhostTemplateEntity->getTemplate(); goto bc398; B689f: $privateKey = $rsaKeyGenerator->generatePrivateKey(); goto F7949; cb9ba: throw new \Exception(implode("\73\40", $errorMessages)); goto a0c69; f1ac0: f5fb1: goto a96a9; B3b79: $csr = $csrGenerator->generate(); goto Fbb00; Adf42: $phpSiteCreator = new PhpSiteCreator($phpSite); goto a7d52; Aad82: $phpSite->setCertificate($certificateEntity); goto Dfdcf; dd9bc: $distinguishedName = new DistinguishedName($domainName, $subjectAlternativeNames); goto Abee9; Fbb00: $selfSignedCertificate = Openssl::createSelfSignedCertificate($privateKey, $csr); goto ecae8; A9b21: $this->siteEntityManager->updateEntity($siteEntity); goto ae309; Dcbe6: if (!(false === empty($varnishCache))) { goto fd989; } goto e699b; Acc29: $phpSiteCreator->createVarnishCacheStructure($varnishCacheSettings); goto f45f6; e3fda: $phpSite->setVhostTemplate($siteEntity->getVhostTemplate()); goto Adf42; ae309: return $siteEntity; goto f72ad; Be352: if (!(false === is_null($subdomain) && "\x77\167\x77" == $subdomain)) { goto C2013; } goto Bd1b3; d8ee2: $siteEntity->setVhostTemplate($vhostTemplate->getContent()); goto C4de5; F7949: $subjectAlternativeNames = []; goto a2883; cb92c: $subdomain = false === empty($subdomain) ? $subdomain : null; goto f94e8; Df697: $phpSite = new PhpSite(); goto C9f65; fe4bf: $phpSite->setRootDirectory($siteEntity->getRootDirectory()); goto Aad82; Fb09a: $defaultVarnishCacheSettings = ["\145\x6e\141\x62\x6c\x65\144" => false, "\x73\x65\x72\166\x65\x72" => self::VARNISH_CACHE_SERVER, "\x63\x61\x63\x68\145\x54\x61\147\x50\x72\x65\x66\x69\x78" => substr(md5(time()), 0, 4)]; goto ebc0c; B71f4: $siteEntity->setCertificate($certificateEntity); goto Df697; C5707: $phpSiteCreator->createLogrotateFile(); goto F4b66; Bf590: $registrableDomain = $resolvedDomainName->registrableDomain()->toString(); goto C69b7; c981b: $siteEntity->setUserPassword($siteUserPassword); goto cb304; C618a: $phpSettingsConstraints = $this->validator->validate($phpSettingsEntity); goto cf10a; c7417: $redirectionVhostTemplate = file_get_contents($redirectionVhostTemplateFile); goto Cbfc3; c4a39: $certificateEntity->setSite($siteEntity); goto b769e; bc398: $siteEntity = $this->siteEntityManager->createEntity(); goto C27e6; D26f4: if (!(true === $varnishCache)) { goto f4c8d; } goto Fd75c; a2883: if (!(true === is_null($subdomain))) { goto b4664; } goto B0fe3; B9ce2: $phpSiteCreator->reloadPhpFpmService(); goto D26f4; E37f3: b4664: goto Be352; d984e: $errorMessages = $this->getConstraintViolationListErrorMessages($constraintViolationList); goto cb9ba; f43f7: ccf46: goto c7417; e193e: if (!(true === is_null($subdomain) || "\x77\167\167" == $subdomain)) { goto f5fb1; } goto f476e; Fe509: $constraintViolationList->addAll($siteConstraints); goto F918a; F2289: $vhostTemplate = new PhpVhostTemplate($phpSite); goto A648d; c5a59: $vhostTemplate->build(); goto d8ee2; D76bd: $phpSiteCreator->createPhpFpmPool(); goto B9ce2; C4de5: $siteEntity->setApplication($vhostTemplateEntity->getName()); goto A9b21; B9f43: $phpSiteCreator->createRootDirectory(); goto C5707; cf170: $phpSite->setUserPassword($siteEntity->getUserPassword()); goto A3d74; a157a: $phpSiteCreator->resetPermissions(); goto F2289; C69b7: $subdomain = $resolvedDomainName->subDomain()->toString(); goto cb92c; d9033: $phpSiteCreator->createPrivateKeyAndCertificate(); goto D76bd; e0fa3: $redirectionVhostTemplateFile = realpath(dirname(__FILE__) . "\57\56\56\x2f\56\x2e\57\x2e\x2e\57\162\x65\163\x6f\165\x72\x63\x65\x73\57\156\147\x69\156\170\57\166\x68\x6f\x73\164\137\164\145\x6d\x70\154\141\164\x65\57\x72\145\x64\151\x72\x65\143\164"); goto d5ced; ce7e2: f4c8d: goto A1b0a; d7d3a: C2013: goto dd9bc; de6b1: $vhostTemplate->addProcessor(new RedirectDomainProcessor()); goto c5a59; A1b0a: $phpSiteCreator->createNginxVhost(); goto D6f3a; A2ce8: $siteEntity->setPhpSettings($phpSettingsEntity); goto Fbbbf; Ab39e: $certificateEntity->setType(CertificateEntity::TYPE_SELF_SIGNED); goto A63da; d1936: $siteConstraints = $this->validator->validate($siteEntity); goto C618a; f72ad: } private function createNodejsSite(string $domainName, string $nodejsVersion, string $port, string $siteUser, string $siteUserPassword) : SiteEntity { goto f25a0; C11d2: $nodejsSiteCreator->createUser(); goto Dbd08; C0a28: $selfSignedCertificate = Openssl::createSelfSignedCertificate($privateKey, $csr); goto C7f28; Cf355: $distinguishedName = new DistinguishedName($domainName, $subjectAlternativeNames); goto A793f; E0eae: $nodejsSiteCreator->createNginxVhost(); goto bdfd1; Ede5f: $siteEntity->setVhostTemplate($vhostTemplate->getContent()); goto f33b5; Bcdac: $siteEntity->setNodejsSettings($nodejsSettingsEntity); goto bc87c; d7a47: $privateKey = $rsaKeyGenerator->generatePrivateKey(); goto e80dc; C60da: $nodejsSite->setUser($siteEntity->getUser()); goto E3520; Fd345: $nodejsSite->setRegistrableDomain($registrableDomain); goto f7e73; b22e0: if (!(false === empty($redirectionVhostTemplate))) { goto f17ad; } goto Cbbed; D6c01: $siteEntity->setUser($siteUser); goto e36f9; Cdc80: $csr = $csrGenerator->generate(); goto C0a28; c6f84: $constraintViolationList = new ConstraintViolationList(); goto f5e3a; e90d0: $nodejsSiteCreator->installNodejs(); goto E7b4d; eafbd: $vhostTemplate->addProcessor(new RedirectServerNameProcessor()); goto Ad5ac; d830c: $siteEntity->setType(SiteEntity::TYPE_NODEJS); goto D3157; bc87c: $nodejsSettingsEntity->setNodejsVersion($nodejsVersion); goto B057a; bdfd1: $nodejsSiteCreator->reloadNginxService(); goto b078c; Ad5ac: $vhostTemplate->addProcessor(new RedirectDomainProcessor()); goto Ce24d; Ea49d: bdbcb: goto e414b; a3d0d: $certificateEntity->setCsr($csr); goto Efec1; e2c45: bc6f2: goto D426b; b078c: $nodejsSiteCreator->resetPermissions(); goto c924c; e081d: $nodejsSiteCreator = new NodejsSiteCreator($nodejsSite); goto C11d2; Db326: if (!(count($siteConstraints) || count($nodejsSettingsConstraints))) { goto bdbcb; } goto c6f84; c843d: if (true === isset($_ENV["\x41\x50\x50\x5f\x48\x54\x54\x50\63"]) && "\164\162\165\145" === $_ENV["\101\120\x50\137\110\x54\124\x50\63"]) { goto B8949; } goto E8aff; F72b1: $siteConstraints = $this->validator->validate($siteEntity); goto F58bf; Be4f1: $nodejsSite = new NodejsSite(); goto C60da; d5c09: if (!(true === is_null($subdomain) || "\x77\x77\167" == $subdomain)) { goto D5836; } goto c843d; f5556: $vhostTemplate->setContent($siteEntity->getVhostTemplate()); goto b89e1; f68fe: $certificateEntity->setCertificate($selfSignedCertificate); goto Edb9a; e80dc: $subjectAlternativeNames = []; goto Fcdb0; E591a: B8949: goto e6961; B3d3b: $siteEntity->setVhostTemplate($vhostTemplate); goto c0281; Ab3fc: $subdomain = false === empty($subdomain) ? $subdomain : null; goto f2819; Ea5bb: $subjectAlternativeNames[] = $registrableDomain; goto c7fc2; f5e3a: $constraintViolationList->addAll($siteConstraints); goto ef2da; D3157: $siteEntity->setDomainName($domainName); goto D6c01; f25a0: $resolvedDomainName = $this->domainNameParser->resolveDomainName($domainName); goto ffc8b; C7f28: $certificateEntity->setDefaultCertificate(true); goto d1bf1; beeb8: $rsaKeyGenerator = new RsaKeyGenerator(); goto d7a47; c0281: $nodejsSettingsEntity = new NodejsSettingsEntity(); goto Bcdac; Ad7a2: $vhostTemplate = $vhostTemplateEntity->getTemplate(); goto Bf7a5; f7e73: $nodejsSite->setSubdomain($subdomain); goto D2526; F05bd: return $siteEntity; goto D8488; B057a: $nodejsSettingsEntity->setPort($port); goto fbb6f; ed945: $siteEntity->setRootDirectory($domainName); goto d5c09; B6e1b: D5836: goto B3d3b; e414b: $certificateEntity = $this->certificateEntityManager->createEntity(); goto bb05a; E8aff: $redirectionVhostTemplateFile = realpath(dirname(__FILE__) . "\x2f\56\56\x2f\56\x2e\x2f\56\56\x2f\x72\145\163\x6f\165\162\x63\145\x73\57\x6e\x67\151\x6e\x78\x2f\166\150\x6f\163\164\137\x74\x65\155\x70\x6c\141\164\145\57\x72\145\x64\x69\162\145\143\x74"); goto B39b4; c7fc2: d9268: goto Cf355; Ce24d: $vhostTemplate->build(); goto Ede5f; e6961: $redirectionVhostTemplateFile = realpath(dirname(__FILE__) . "\x2f\x2e\x2e\57\56\56\57\56\56\57\x72\145\x73\x6f\165\162\143\x65\163\57\156\147\x69\x6e\170\57\166\x68\157\163\164\137\x74\x65\155\x70\x6c\141\x74\145\x2f\162\145\x64\151\x72\x65\143\x74\55\150\164\164\160\x33"); goto F072f; A5d94: $vhostTemplate->addProcessor(new ServerNameProcessor()); goto eafbd; E0f11: $nodejsSite->setDomainName($domainName); goto Fd345; Dc944: $subjectAlternativeNames[] = sprintf("\x77\x77\167\56\45\x73", $domainName); goto e2c45; Efec1: $certificateEntity->setPrivateKey($privateKey->getPEM()); goto f68fe; fbb6f: $nodejsSettingsEntity->setSite($siteEntity); goto F72b1; f33b5: $siteEntity->setApplication($vhostTemplateEntity->getName()); goto D8101; Dbd08: $nodejsSiteCreator->createRootDirectory(); goto e8670; cb5e1: $redirectionVhostTemplate = file_get_contents($redirectionVhostTemplateFile); goto b22e0; A793f: $csrGenerator = new CsrGenerator($privateKey, $distinguishedName); goto Cdc80; b89e1: $vhostTemplate->resetProcessors(); goto A5d94; Abdf2: f17ad: goto B6e1b; bb05a: $certificateEntity->setSite($siteEntity); goto beeb8; D426b: if (!(false === is_null($subdomain) && "\x77\x77\167" == $subdomain)) { goto d9268; } goto Ea5bb; Cbe9a: $errorMessages = $this->getConstraintViolationListErrorMessages($constraintViolationList); goto E5d7f; e0710: $subdomain = $resolvedDomainName->subDomain()->toString(); goto Ab3fc; E3520: $nodejsSite->setUserPassword($siteEntity->getUserPassword()); goto E0f11; D843e: $nodejsSite->setNodejsSettings($nodejsSettingsEntity); goto F613a; F072f: a8a1c: goto cb5e1; E5d7f: throw new \Exception(implode("\73\40", $errorMessages)); goto Ea49d; e8670: $nodejsSiteCreator->createLogrotateFile(); goto d8e31; Fcdb0: if (!(true === is_null($subdomain))) { goto bc6f2; } goto Dc944; d1bf1: $certificateEntity->setType(CertificateEntity::TYPE_SELF_SIGNED); goto a3d0d; Bf7a5: $siteEntity = $this->siteEntityManager->createEntity(); goto d830c; d8e31: $nodejsSiteCreator->createNvmDirectory(); goto e90d0; Edb9a: $siteEntity->setCertificate($certificateEntity); goto Be4f1; c6ca2: $nodejsSite->setCertificate($certificateEntity); goto D843e; E7b4d: $nodejsSiteCreator->createPrivateKeyAndCertificate(); goto E0eae; B39b4: goto a8a1c; goto E591a; D8101: $this->siteEntityManager->updateEntity($siteEntity); goto F05bd; ffc8b: $registrableDomain = $resolvedDomainName->registrableDomain()->toString(); goto e0710; e36f9: $siteEntity->setUserPassword($siteUserPassword); goto ed945; ef2da: $constraintViolationList->addAll($nodejsSettingsConstraints); goto Cbe9a; Cbbed: $vhostTemplate = sprintf("\45\163\45\163", $redirectionVhostTemplate, $vhostTemplate); goto Abdf2; D2526: $nodejsSite->setRootDirectory($siteEntity->getRootDirectory()); goto c6ca2; c924c: $vhostTemplate = new StaticVhostTemplate($nodejsSite); goto f5556; F58bf: $nodejsSettingsConstraints = $this->validator->validate($nodejsSettingsEntity); goto Db326; f2819: $vhostTemplateEntity = $this->getVhostTemplateEntity(self::NODEJS_VHOST_TEMPLATE_NAME); goto Ad7a2; F613a: $nodejsSite->setVhostTemplate($siteEntity->getVhostTemplate()); goto e081d; D8488: } private function createPythonSite(string $domainName, string $pythonVersion, string $port, string $siteUser, string $siteUserPassword) : SiteEntity { goto a74fd; ebc14: if (true === isset($_ENV["\x41\120\120\x5f\x48\x54\124\x50\63"]) && "\164\x72\x75\x65" === $_ENV["\101\x50\120\137\110\124\x54\x50\63"]) { goto ad3c6; } goto Ae0a0; fb450: $siteEntity->setDomainName($domainName); goto abc0e; e3a69: $pythonSite->setUserPassword($siteEntity->getUserPassword()); goto Ec6d4; Aa245: $certificateEntity->setCsr($csr); goto c8d3c; efde5: $registrableDomain = $resolvedDomainName->registrableDomain()->toString(); goto dad63; d4fb8: $distinguishedName = new DistinguishedName($domainName, $subjectAlternativeNames); goto a6f52; Ae0a0: $redirectionVhostTemplateFile = realpath(dirname(__FILE__) . "\x2f\x2e\56\57\56\56\57\56\56\x2f\162\145\163\x6f\165\x72\x63\145\x73\57\x6e\x67\151\156\x78\57\x76\x68\157\163\164\137\164\x65\155\x70\154\141\164\145\57\x72\145\144\151\x72\x65\143\x74"); goto d84ac; a74fd: $resolvedDomainName = $this->domainNameParser->resolveDomainName($domainName); goto efde5; d4a8d: $pythonSite->setRegistrableDomain($registrableDomain); goto f901e; A625f: $vhostTemplate->addProcessor(new ServerNameProcessor()); goto e81dc; ca649: $pythonSiteCreator->createRootDirectory(); goto e9b82; E87ce: $this->siteEntityManager->updateEntity($siteEntity); goto C04ac; f9345: $pythonSiteCreator->resetPermissions(); goto ebd86; Beb96: $siteEntity = $this->siteEntityManager->createEntity(); goto d93be; B0661: f93d2: goto ff9a7; e81dc: $vhostTemplate->addProcessor(new RedirectServerNameProcessor()); goto c285e; E0916: return $siteEntity; goto F866c; Fa968: be031: goto F829f; b660c: if (!(false === empty($redirectionVhostTemplate))) { goto D2655; } goto Db64b; c285e: $vhostTemplate->addProcessor(new RedirectDomainProcessor()); goto A1aff; Afe95: Ba370: goto C9a8f; fa7fb: D2655: goto fae80; d67ac: $constraintViolationList->addAll($siteConstraints); goto Fb7cf; efdc7: if (!(true === is_null($subdomain))) { goto Ba370; } goto Ffa59; Fc774: $pythonSettingsEntity->setSite($siteEntity); goto b0c32; cede2: $pythonSiteCreator->createPrivateKeyAndCertificate(); goto a9c4f; Fafb2: $errorMessages = $this->getConstraintViolationListErrorMessages($constraintViolationList); goto B8cdd; D4a36: $pythonSite->setCertificate($certificateEntity); goto C21ef; C21ef: $pythonSite->setPythonSettings($pythonSettingsEntity); goto f63a0; f1f65: $siteEntity->setUserPassword($siteUserPassword); goto E4bc6; E2d2c: ad3c6: goto F96d0; b077f: $rsaKeyGenerator = new RsaKeyGenerator(); goto d0c3b; F4e56: Bfc8a: goto d4fb8; f63a0: $pythonSite->setVhostTemplate($siteEntity->getVhostTemplate()); goto C8994; F4f36: $pythonSite->setRootDirectory($siteEntity->getRootDirectory()); goto D4a36; Bc315: if (!(true === is_null($subdomain) || "\167\167\167" == $subdomain)) { goto cfe3d; } goto ebc14; B140a: $siteEntity->setPythonSettings($pythonSettingsEntity); goto Ef811; B8cdd: throw new \Exception(implode("\73\x20", $errorMessages)); goto Fa968; C8994: $pythonSiteCreator = new PythonSiteCreator($pythonSite); goto F67cc; E72a9: $pythonSettingsEntity->setPort($port); goto Fc774; ebd86: $vhostTemplate = new PythonVhostTemplate($pythonSite); goto b15a7; c8d3c: $certificateEntity->setPrivateKey($privateKey->getPEM()); goto D41b3; F829f: $certificateEntity = $this->certificateEntityManager->createEntity(); goto C4582; Ea57d: $siteEntity->setVhostTemplate($vhostTemplate->getContent()); goto E87ce; A1aff: $vhostTemplate->build(); goto Ea57d; fdad0: $vhostTemplate->resetProcessors(); goto A625f; F96d0: $redirectionVhostTemplateFile = realpath(dirname(__FILE__) . "\57\x2e\x2e\57\56\56\57\56\56\x2f\162\x65\x73\x6f\165\x72\143\145\163\x2f\156\x67\x69\x6e\x78\57\x76\150\157\163\x74\137\x74\145\155\160\154\141\x74\145\x2f\162\145\x64\151\x72\145\x63\x74\55\150\164\164\x70\63"); goto B0661; e9bec: $certificateEntity->setDefaultCertificate(true); goto D447c; Fa658: $pythonSite->setUser($siteEntity->getUser()); goto e3a69; d93be: $siteEntity->setType(SiteEntity::TYPE_PYTHON); goto fb450; c3313: $vhostTemplateEntity = $this->getVhostTemplateEntity(self::PYTHON_VHOST_TEMPLATE_NAME); goto b3934; E740e: $subdomain = false === empty($subdomain) ? $subdomain : null; goto c3313; D447c: $certificateEntity->setType(CertificateEntity::TYPE_SELF_SIGNED); goto Aa245; F5c48: $subjectAlternativeNames[] = $registrableDomain; goto F4e56; C4582: $certificateEntity->setSite($siteEntity); goto b077f; dad63: $subdomain = $resolvedDomainName->subDomain()->toString(); goto E740e; Ec6d4: $pythonSite->setDomainName($domainName); goto d4a8d; b348d: $pythonSettingsConstraints = $this->validator->validate($pythonSettingsEntity); goto a2a54; cbfd7: $pythonSite = new PythonSite(); goto Fa658; ff9a7: $redirectionVhostTemplate = file_get_contents($redirectionVhostTemplateFile); goto b660c; e9b82: $pythonSiteCreator->createLogrotateFile(); goto A0e03; b0c32: $siteConstraints = $this->validator->validate($siteEntity); goto b348d; D41b3: $certificateEntity->setCertificate($selfSignedCertificate); goto f6f70; Fb7cf: $constraintViolationList->addAll($pythonSettingsConstraints); goto Fafb2; F67cc: $pythonSiteCreator->createUser(); goto ca649; fae80: cfe3d: goto Bbafa; ed4b0: $constraintViolationList = new ConstraintViolationList(); goto d67ac; Bbafa: $siteEntity->setVhostTemplate($vhostTemplate); goto a3004; C9a8f: if (!(false === is_null($subdomain) && "\x77\167\167" == $subdomain)) { goto Bfc8a; } goto F5c48; A0e03: $pythonSiteCreator->writePythonVersionFile(); goto cede2; Ffa59: $subjectAlternativeNames[] = sprintf("\167\x77\x77\x2e\45\x73", $domainName); goto Afe95; f6f70: $siteEntity->setCertificate($certificateEntity); goto cbfd7; a9c4f: $pythonSiteCreator->createNginxVhost(); goto F92f2; a6f52: $csrGenerator = new CsrGenerator($privateKey, $distinguishedName); goto d534c; b3934: $vhostTemplate = $vhostTemplateEntity->getTemplate(); goto Beb96; F92f2: $pythonSiteCreator->reloadNginxService(); goto f9345; abc0e: $siteEntity->setUser($siteUser); goto f1f65; Ef811: $pythonSettingsEntity->setPythonVersion($pythonVersion); goto E72a9; b15a7: $vhostTemplate->setContent($siteEntity->getVhostTemplate()); goto fdad0; a2a54: if (!(count($siteConstraints) || count($pythonSettingsConstraints))) { goto be031; } goto ed4b0; C04ac: $siteEntity->setApplication($vhostTemplateEntity->getName()); goto E0916; a3004: $pythonSettingsEntity = new PythonSettingsEntity(); goto B140a; E4bc6: $siteEntity->setRootDirectory($domainName); goto Bc315; Db64b: $vhostTemplate = sprintf("\x25\x73\x25\163", $redirectionVhostTemplate, $vhostTemplate); goto fa7fb; Bedec: $selfSignedCertificate = Openssl::createSelfSignedCertificate($privateKey, $csr); goto e9bec; d84ac: goto f93d2; goto E2d2c; d534c: $csr = $csrGenerator->generate(); goto Bedec; f901e: $pythonSite->setSubdomain($subdomain); goto F4f36; d0c3b: $privateKey = $rsaKeyGenerator->generatePrivateKey(); goto D2a80; D2a80: $subjectAlternativeNames = []; goto efdc7; F866c: } private function createStaticSite(string $domainName, string $siteUser, string $siteUserPassword) : SiteEntity { goto Bd058; E0522: $siteEntity->setUser($siteUser); goto c8666; C5fbe: $siteEntity->setApplication($vhostTemplateEntity->getName()); goto E4be6; F57c8: $certificateEntity->setCsr($csr); goto e05d8; A01a0: $certificateEntity->setCertificate($selfSignedCertificate); goto D0287; Eff1b: $privateKey = $rsaKeyGenerator->generatePrivateKey(); goto F7f4e; Bca07: $redirectionVhostTemplateFile = realpath(dirname(__FILE__) . "\57\56\56\x2f\x2e\56\x2f\x2e\x2e\x2f\x72\x65\163\x6f\165\x72\143\145\x73\57\156\147\x69\x6e\170\x2f\x76\150\157\x73\164\137\164\x65\155\160\154\141\164\145\57\x72\145\x64\151\162\x65\143\164\x2d\150\164\164\160\x33"); goto be703; De8fa: $vhostTemplateEntity = $this->getVhostTemplateEntity(self::STATIC_SITE_VHOST_TEMPLATE_NAME); goto aab6c; Bd058: $resolvedDomainName = $this->domainNameParser->resolveDomainName($domainName); goto Ce570; B3b61: $staticSite->setUser($siteEntity->getUser()); goto C2b18; F9aac: if (!(false === empty($redirectionVhostTemplate))) { goto Ddf1d; } goto fac9b; c8666: $siteEntity->setUserPassword($siteUserPassword); goto d700b; Efcb4: f60ab: goto e4d17; B6f12: $staticSiteCreator->createUser(); goto C607f; d44ad: $staticSiteCreator = new StaticSiteCreator($staticSite); goto B6f12; f7e0c: $certificateEntity->setType(CertificateEntity::TYPE_SELF_SIGNED); goto F57c8; C747a: $constraintViolationList->addAll($siteConstraints); goto B1c45; a838d: $certificateEntity->setSite($siteEntity); goto B3f96; cfbe9: $staticSiteCreator->createNginxVhost(); goto c4d32; c4d32: $staticSiteCreator->reloadNginxService(); goto Adeae; Dfdb1: $subjectAlternativeNames[] = sprintf("\x77\167\x77\56\x25\x73", $domainName); goto c90aa; fac9b: $vhostTemplate = sprintf("\x25\x73\x25\163", $redirectionVhostTemplate, $vhostTemplate); goto A6b5e; ae643: $staticSiteCreator->createLogrotateFile(); goto ffe1e; e9637: if (!(true === is_null($subdomain) || "\x77\167\x77" == $subdomain)) { goto C009f; } goto e6c13; d94e9: $vhostTemplate->addProcessor(new RedirectServerNameProcessor()); goto d588b; B136f: $staticSite = new StaticSite(); goto B3b61; Ee469: $staticSite->setSubdomain($subdomain); goto B175f; Ce570: $registrableDomain = $resolvedDomainName->registrableDomain()->toString(); goto Fb823; c90aa: E519a: goto Eb065; efc8f: $selfSignedCertificate = Openssl::createSelfSignedCertificate($privateKey, $csr); goto d8ec7; e1980: $siteEntity->setVhostTemplate($vhostTemplate->getContent()); goto C5fbe; C2b18: $staticSite->setUserPassword($siteEntity->getUserPassword()); goto d5854; B3a8f: $siteConstraints = $this->validator->validate($siteEntity); goto f4e1f; F7f4e: $subjectAlternativeNames = []; goto F2c11; C607f: $staticSiteCreator->createRootDirectory(); goto ae643; D12f3: $redirectionVhostTemplateFile = realpath(dirname(__FILE__) . "\x2f\56\x2e\x2f\x2e\56\57\x2e\56\x2f\x72\145\x73\x6f\x75\162\x63\145\163\x2f\x6e\147\x69\x6e\x78\x2f\x76\x68\x6f\163\x74\x5f\x74\x65\155\x70\x6c\x61\164\x65\x2f\162\145\x64\151\162\145\x63\164"); goto a8747; e05d8: $certificateEntity->setPrivateKey($privateKey->getPEM()); goto A01a0; A6b5e: Ddf1d: goto cd2dc; e859c: $siteEntity->setVhostTemplate($vhostTemplate); goto B3a8f; D4664: $constraintViolationList = new ConstraintViolationList(); goto C747a; E4be6: $this->siteEntityManager->updateEntity($siteEntity); goto Bf6e6; B474f: $subdomain = false === empty($subdomain) ? $subdomain : null; goto De8fa; B3f96: $rsaKeyGenerator = new RsaKeyGenerator(); goto Eff1b; E47ba: $siteEntity->setType(SiteEntity::TYPE_STATIC); goto Dd68c; A7ea0: $vhostTemplate->addProcessor(new ServerNameProcessor()); goto d94e9; a8747: goto F7917; goto ac84b; f508d: $siteEntity = $this->siteEntityManager->createEntity(); goto E47ba; d6f8d: $distinguishedName = new DistinguishedName($domainName, $subjectAlternativeNames); goto E20a1; d8ec7: $certificateEntity->setDefaultCertificate(true); goto f7e0c; d5854: $staticSite->setDomainName($domainName); goto Eb001; Fb823: $subdomain = $resolvedDomainName->subDomain()->toString(); goto B474f; A01c2: $staticSite->setCertificate($certificateEntity); goto D3001; d169b: $vhostTemplate->setContent($siteEntity->getVhostTemplate()); goto D9527; cd2dc: C009f: goto e859c; be703: F7917: goto fd0ea; D9527: $vhostTemplate->resetProcessors(); goto A7ea0; D3001: $staticSite->setVhostTemplate($siteEntity->getVhostTemplate()); goto d44ad; Be1a8: $subjectAlternativeNames[] = $registrableDomain; goto b4881; D7f2a: $vhostTemplate->build(); goto e1980; B175f: $staticSite->setRootDirectory($siteEntity->getRootDirectory()); goto A01c2; Ec140: $csr = $csrGenerator->generate(); goto efc8f; C4524: throw new \Exception(implode("\x3b\40", $errorMessages)); goto Efcb4; e4d17: $certificateEntity = $this->certificateEntityManager->createEntity(); goto a838d; c2f37: $staticSiteCreator->createPrivateKeyAndCertificate(); goto cfbe9; ac84b: bc0ed: goto Bca07; E20a1: $csrGenerator = new CsrGenerator($privateKey, $distinguishedName); goto Ec140; Bf6e6: return $siteEntity; goto e99c1; D0287: $siteEntity->setCertificate($certificateEntity); goto B136f; b4881: A7dd7: goto d6f8d; aab6c: $vhostTemplate = $vhostTemplateEntity->getTemplate(); goto f508d; ffe1e: $staticSiteCreator->createIndexHtml(); goto c2f37; d588b: $vhostTemplate->addProcessor(new RedirectDomainProcessor()); goto D7f2a; Adeae: $staticSiteCreator->resetPermissions(); goto Fa5d3; B1c45: $errorMessages = $this->getConstraintViolationListErrorMessages($constraintViolationList); goto C4524; Eb001: $staticSite->setRegistrableDomain($registrableDomain); goto Ee469; Eb065: if (!(false === is_null($subdomain) && "\x77\167\x77" == $subdomain)) { goto A7dd7; } goto Be1a8; f4e1f: if (!count($siteConstraints)) { goto f60ab; } goto D4664; e6c13: if (true === isset($_ENV["\x41\120\120\x5f\110\124\x54\120\63"]) && "\164\162\165\145" === $_ENV["\101\120\x50\137\x48\124\x54\x50\x33"]) { goto bc0ed; } goto D12f3; fd0ea: $redirectionVhostTemplate = file_get_contents($redirectionVhostTemplateFile); goto F9aac; F2c11: if (!(true === is_null($subdomain))) { goto E519a; } goto Dfdb1; Dd68c: $siteEntity->setDomainName($domainName); goto E0522; Fa5d3: $vhostTemplate = new StaticVhostTemplate($staticSite); goto d169b; d700b: $siteEntity->setRootDirectory($domainName); goto e9637; e99c1: } private function createReverseProxySite(string $domainName, string $reverseProxyUrl, string $siteUser, string $siteUserPassword) : SiteEntity { goto e0115; E8a76: $siteEntity->setVhostTemplate($vhostTemplate->getContent()); goto Ca8b8; f9f09: $reversProxySiteCreator = new ReverseProxySiteCreator($reverseProxySite); goto bd786; Eccae: $reversProxySiteCreator->createPrivateKeyAndCertificate(); goto C4ab7; d21e9: $reversProxySiteCreator->createRootDirectory(); goto Ebc9b; F64c0: $csrGenerator = new CsrGenerator($privateKey, $distinguishedName); goto a0279; b945c: $redirectionVhostTemplateFile = realpath(dirname(__FILE__) . "\x2f\x2e\x2e\57\56\x2e\x2f\56\x2e\x2f\162\145\163\x6f\165\x72\143\x65\x73\57\156\147\x69\x6e\170\x2f\x76\x68\x6f\x73\x74\x5f\x74\x65\x6d\160\x6c\x61\x74\145\57\162\x65\x64\151\162\x65\143\164\55\x68\164\x74\x70\x33"); goto f6510; aadec: $certificateEntity->setDefaultCertificate(true); goto a8002; e45aa: if (!(true === is_null($subdomain) || "\x77\167\167" == $subdomain)) { goto a7efc; } goto ef103; C40bb: $constraintViolationList = new ConstraintViolationList(); goto D20f1; Fec40: $distinguishedName = new DistinguishedName($domainName, $subjectAlternativeNames); goto F64c0; bf993: $subjectAlternativeNames[] = sprintf("\x77\x77\167\56\x25\x73", $domainName); goto b232a; fa4e1: if (!(false === empty($redirectionVhostTemplate))) { goto e0dfe; } goto fd5d1; a520e: $siteEntity = $this->siteEntityManager->createEntity(); goto D4390; e0331: $siteEntity->setCertificate($certificateEntity); goto A9833; Da176: $reverseProxySite->setRootDirectory($siteEntity->getRootDirectory()); goto d2ec8; f2725: $subdomain = $resolvedDomainName->subDomain()->toString(); goto a025e; cc2f6: if (!(true === is_null($subdomain))) { goto af85f; } goto bf993; ee04a: if (!count($siteConstraints)) { goto a48d9; } goto C40bb; ee416: $siteEntity->setVhostTemplate($vhostTemplate); goto c8d7e; a0279: $csr = $csrGenerator->generate(); goto F6e54; c224b: $vhostTemplate = new ReverseProxyVhostTemplate($reverseProxySite); goto c820f; cb429: $siteEntity->setRootDirectory($domainName); goto c2bd2; C4ab7: $reversProxySiteCreator->createNginxVhost(); goto fdc2c; D0a31: $redirectionVhostTemplate = file_get_contents($redirectionVhostTemplateFile); goto fa4e1; Dc0e0: $vhostTemplate->addProcessor(new RedirectServerNameProcessor()); goto afb75; fdc2c: $reversProxySiteCreator->reloadNginxService(); goto F9ed7; Ebc9b: $reversProxySiteCreator->createLogrotateFile(); goto Eccae; e2476: $certificateEntity->setSite($siteEntity); goto e1f5e; B382e: $redirectionVhostTemplateFile = realpath(dirname(__FILE__) . "\57\56\56\57\56\x2e\x2f\x2e\x2e\57\x72\145\x73\x6f\165\x72\x63\x65\x73\57\x6e\147\151\x6e\x78\x2f\x76\x68\157\x73\164\137\164\145\x6d\160\154\141\164\145\x2f\162\x65\x64\x69\162\x65\x63\x74"); goto Dd102; b232a: af85f: goto A4c79; F6e54: $selfSignedCertificate = Openssl::createSelfSignedCertificate($privateKey, $csr); goto aadec; F108f: $vhostTemplate->resetProcessors(); goto e3bd0; Da1fd: $reverseProxySite->setUser($siteEntity->getUser()); goto f623f; F9ed7: $reversProxySiteCreator->resetPermissions(); goto c224b; e3bd0: $vhostTemplate->addProcessor(new ServerNameProcessor()); goto Dc0e0; c820f: $vhostTemplate->setContent($siteEntity->getVhostTemplate()); goto F108f; a025e: $subdomain = false === empty($subdomain) ? $subdomain : null; goto e7e66; D20f1: $constraintViolationList->addAll($siteConstraints); goto F5bf8; F9499: $certificateEntity->setCertificate($selfSignedCertificate); goto e0331; bc6b5: $subjectAlternativeNames[] = $registrableDomain; goto ed4f0; B939e: $subjectAlternativeNames = []; goto cc2f6; b92f5: $certificateEntity = $this->certificateEntityManager->createEntity(); goto e2476; b1892: $certificateEntity->setCsr($csr); goto D4a51; F5e85: $privateKey = $rsaKeyGenerator->generatePrivateKey(); goto B939e; ef103: if (true === isset($_ENV["\101\120\120\137\110\x54\x54\x50\63"]) && "\x74\x72\165\145" === $_ENV["\101\x50\120\137\110\124\124\120\63"]) { goto e8bb7; } goto B382e; A1e7d: $reverseProxySite->setSubdomain($subdomain); goto Da176; F5855: e0dfe: goto Cfca7; c2bd2: $siteEntity->setReverseProxyUrl($reverseProxyUrl); goto e45aa; ea374: $vhostTemplate = $vhostTemplateEntity->getTemplate(); goto a520e; A9833: $reverseProxySite = new ReverseProxySite(); goto Da1fd; e55b4: return $siteEntity; goto Be193; f623f: $reverseProxySite->setUserPassword($siteEntity->getUserPassword()); goto C6174; Dd102: goto Cbd30; goto d067a; Ca8b8: $siteEntity->setApplication($vhostTemplateEntity->getName()); goto cefb7; c8d7e: $siteConstraints = $this->validator->validate($siteEntity); goto ee04a; cefb7: $this->siteEntityManager->updateEntity($siteEntity); goto e55b4; Bdece: $siteEntity->setUserPassword($siteUserPassword); goto cb429; ed4f0: A7019: goto Fec40; d067a: e8bb7: goto b945c; D4390: $siteEntity->setType(SiteEntity::TYPE_REVERSE_PROXY); goto b49eb; F28b7: a48d9: goto b92f5; A6dce: $vhostTemplate->build(); goto E8a76; Cfca7: a7efc: goto ee416; bd786: $reversProxySiteCreator->createUser(); goto d21e9; D4a51: $certificateEntity->setPrivateKey($privateKey->getPEM()); goto F9499; d2ec8: $reverseProxySite->setCertificate($certificateEntity); goto C5952; C6174: $reverseProxySite->setDomainName($domainName); goto e9005; e0115: $resolvedDomainName = $this->domainNameParser->resolveDomainName($domainName); goto E0443; E0443: $registrableDomain = $resolvedDomainName->registrableDomain()->toString(); goto f2725; b49eb: $siteEntity->setDomainName($domainName); goto C31f7; C31f7: $siteEntity->setUser($siteUser); goto Bdece; fd5d1: $vhostTemplate = sprintf("\45\163\45\163", $redirectionVhostTemplate, $vhostTemplate); goto F5855; Abb3c: throw new \Exception(implode("\x3b\40", $errorMessages)); goto F28b7; e1f5e: $rsaKeyGenerator = new RsaKeyGenerator(); goto F5e85; e7e66: $vhostTemplateEntity = $this->getVhostTemplateEntity(self::REVERSE_PROXY_VHOST_TEMPLATE_NAME); goto ea374; A4c79: if (!(false === is_null($subdomain) && "\x77\x77\x77" == $subdomain)) { goto A7019; } goto bc6b5; C5952: $reverseProxySite->setReverseProxyUrl($reverseProxyUrl); goto E2f5e; f6510: Cbd30: goto D0a31; a8002: $certificateEntity->setType(CertificateEntity::TYPE_SELF_SIGNED); goto b1892; e9005: $reverseProxySite->setRegistrableDomain($registrableDomain); goto A1e7d; E2f5e: $reverseProxySite->setVhostTemplate($siteEntity->getVhostTemplate()); goto f9f09; F5bf8: $errorMessages = $this->getConstraintViolationListErrorMessages($constraintViolationList); goto Abb3c; afb75: $vhostTemplate->addProcessor(new RedirectDomainProcessor()); goto A6dce; Be193: } public function installedWordPressSite(Request $request) : Response { goto E81d3; Dfda6: b1c16: goto E2c61; f71c0: F3e60: goto fd89f; E2c61: return $response; goto A7518; Bc653: goto b1c16; goto f71c0; aef42: $siteCredentials = $session->get("\163\151\164\x65\103\162\x65\x64\x65\156\164\151\141\154\163"); goto ac8bb; ac8bb: if (false === empty($siteCredentials)) { goto F3e60; } goto Fd567; F920b: $response = $this->render("\106\162\x6f\x6e\164\x65\x6e\x64\57\123\151\164\145\57\x4e\145\167\57\x77\x6f\162\x64\160\x72\x65\163\163\x2d\x69\x6e\163\164\141\x6c\x6c\145\144\x2e\150\164\155\154\x2e\x74\167\x69\147", ["\163\x69\x74\x65\103\162\x65\x64\145\156\x74\x69\141\154\x73" => $siteCredentials]); goto a34bb; fd89f: $siteCredentials = $this->renderSiteCredentials($siteCredentials); goto F920b; a34bb: return $response; goto Dfda6; Fd567: $response = $this->redirect($this->generateUrl("\x63\154\x70\x5f\163\151\164\145\x73")); goto Bc653; E81d3: $session = $request->getSession(); goto aef42; A7518: } private function renderSiteCredentials(array $siteCredentials) : string { goto bcd83; b1d7a: return $renderedCredentials; goto Ae8b5; A433a: acf38: goto b1d7a; bcd83: $renderedCredentials = ''; goto ebbb6; ebbb6: foreach ($siteCredentials as $section => $sectionData) { goto e2bbd; Eb527: a7a95: goto b391b; C4501: b40ef: goto a0d6c; a02b1: $renderedCredentials .= "\x2d\x2d\55\55\55\x2d\55\55\55\x2d\x2d\x2d\x2d\55\55\x2d\55\55\55\x2d\x2d\55\x2d\55\55\x2d\55\x2d\55\x2d\x2d\x2d\55\x2d\55\55\x2d\55\55\x2d\x2d\x2d\55\x2d\x2d\x2d\x2d\55\x2d\x2d\55\x2d\x2d\55" . PHP_EOL; goto Bc742; a0d6c: $renderedCredentials .= PHP_EOL; goto Eb527; e2bbd: $renderedCredentials .= $section . PHP_EOL; goto a02b1; Bc742: foreach ($sectionData as $key => $value) { $renderedCredentials .= sprintf("\45\x73\x3a\x20\45\163", $key, $value) . PHP_EOL; D9c32: } goto C4501; b391b: } goto A433a; Ae8b5: } private function getVhostTemplateEntity(string $name) : mixed { goto F712f; Bad65: if (!(true === is_null($vhostTemplateEntity))) { goto ada4b; } goto C0568; F712f: $vhostTemplateEntity = $this->vhostTemplateEntityManager->findOneByName($name); goto Bad65; c772d: return $vhostTemplateEntity; goto b928e; cef7e: ada4b: goto c772d; C0568: throw new \Exception($this->translator->trans("\126\150\157\163\164\x20\x54\145\155\160\x6c\x61\164\x65\40\x25\166\x68\x6f\x73\x74\124\145\155\160\154\141\164\x65\45\40\144\x6f\145\x73\40\156\x6f\x74\40\x65\x78\x69\163\164\x2e", ["\x25\166\x68\x6f\x73\x74\x54\145\155\x70\154\x61\x74\145\x25" => $name])); goto cef7e; b928e: } private function getConstraintViolationListErrorMessages(ConstraintViolationList $constraintViolationList) : array { goto fe151; ad461: ad6bd: goto a4cd7; a4cd7: return $errorMessages; goto bca18; fe151: $errorMessages = []; goto be3ea; be3ea: foreach ($constraintViolationList as $constraint) { $errorMessages[] = sprintf("\45\x73\72\x20\45\x73", $constraint->getPropertyPath(), $constraint->getMessage()); B59bb: } goto ad461; bca18: } private function generateNameFromDomainName(string $domainName) : string { goto cd906; fad0f: $explodedSubdomain = explode("\56", $subdomain); goto Fbb4c; Fbb4c: $name = sprintf("\45\163\x2d\x25\x73", $explodedRegistrableDomain[0] ?? '', implode("\55", $explodedSubdomain)); goto D198b; b821b: return $name; goto E65a1; cbeea: if (!(false === empty($subdomain))) { goto d779f; } goto fad0f; A715d: if ("\x77\167\167" == $subdomain) { goto c2151; } goto dff2b; A774a: $explodedRegistrableDomain = explode("\x2e", $registrableDomain); goto A715d; D198b: d779f: goto Edea7; E318f: A864b: goto b821b; Ae442: $name = $explodedRegistrableDomain[0] ?? ''; goto E318f; c333c: $subdomain = $resolvedDomainName->subDomain()->toString(); goto A774a; e4619: $registrableDomain = $resolvedDomainName->registrableDomain()->toString(); goto c333c; cd906: $resolvedDomainName = $this->domainNameParser->resolveDomainName($domainName); goto e4619; dff2b: $name = $explodedRegistrableDomain[0] ?? ''; goto cbeea; f6945: c2151: goto Ae442; Edea7: goto A864b; goto f6945; E65a1: } private function getUniqueDatabaseName(string $databaseName) : string { goto d4d61; B9e5e: $uniqueDatabaseName = sprintf("\x25\x73\55\x64\x62", $uniqueDatabaseName); goto a08c8; A6ad9: $activeDatabaseServerEntity = $this->databaseServerEntityManager->getActiveDatabaseServer(); goto b1375; A62d6: $i++; goto B07a5; E9bf0: D2f63: goto e6c20; aea63: $databases = $databaseConnection->getDatabases(); goto f50c4; c1f91: return $uniqueDatabaseName; goto B4f47; Cfe04: $i = 0; goto A6ad9; e6c20: $databaseExists = in_array($uniqueDatabaseName, $databases); goto A62d6; D724b: if (!(strlen($uniqueDatabaseName) <= 3)) { goto a36d5; } goto B9e5e; B000d: if (!(0 < $i)) { goto D2f63; } goto D62be; f50c4: c3497: goto B000d; a08c8: a36d5: goto Cfe04; D62be: $uniqueDatabaseName = sprintf("\x25\163\x2d\x25\x73", $databaseName, $i); goto E9bf0; d4d61: $uniqueDatabaseName = $databaseName; goto D724b; B4a31: a8561: goto c1f91; b1375: $databaseConnection = new DatabaseConnection($activeDatabaseServerEntity); goto aea63; B07a5: if (true === $databaseExists) { goto c3497; } goto B4a31; B4f47: } private function getUniqueDatabaseUserName(string $databaseUserName) : string { goto a08eb; C49f7: if (!(strlen($uniqueDatabaseUserName) <= 3)) { goto cc780; } goto d2f19; fcdfb: b86ad: goto Bd67e; a08eb: $uniqueDatabaseUserName = $databaseUserName; goto C49f7; B4744: $uniqueDatabaseUserName = sprintf("\45\x73\55\x25\163", $databaseUserName, $i); goto fcdfb; A39a5: $i++; goto f1b7f; f1b7f: if (false === is_null($databaseUserEntity)) { goto b440f; } goto Ed593; Bd67e: $databaseUserEntity = $this->databaseUserEntityManager->findOneByUserName($uniqueDatabaseUserName); goto A39a5; c9893: $i = 0; goto Eb14f; Ed593: Bee3f: goto d19cd; Eb14f: b440f: goto a87f6; d19cd: $uniqueDatabaseUserName = mb_substr($uniqueDatabaseUserName, 0, 32); goto B8512; d2f19: $uniqueDatabaseUserName = sprintf("\45\163\x2d\x64\x62", $uniqueDatabaseUserName); goto c508f; a87f6: if (!(0 < $i)) { goto b86ad; } goto B4744; c508f: cc780: goto c9893; B8512: return $uniqueDatabaseUserName; goto f8471; f8471: } private function hasPermissions() : bool { goto B0b2f; f0fc7: b1767: goto D6f5b; B0b2f: $user = $this->getUser(); goto fee84; D98e7: return true; goto f0fc7; fee84: if (!(UserEntity::ROLE_USER != $user->getRole())) { goto b1767; } goto D98e7; D6f5b: return false; goto a5609; a5609: } }
